#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cti_io.h"


/* This files contains the main routines that are needed 
   to read CTI format scn files (usually sinograms and 
   to write CTI img files (usually reconstructions) 
*/


/* ******************************************************/
/***  ROUTINES FOR READING CTI SCAN FILES *************/
/********************************************************/

/* This routine is a specialized top-level routine to read the 
   detector normalization sinogram containing only a single frame 
   input  : filename of normalization file (scn-file) , plane to retrieve
   returns: 56 by 112 double array conatining the normalization 
            sinogram for plane 
*/
double** read_nrm_plane(char *fname, int plane, int DEADTIME_FLAG)
{
  FILE *fp;
  struct main_head mh;
  struct scn_head sh;
  long ****img_code; /* img_code[plane][gate][frame][bed] */
  int BYTE_ORDER,stat;
  double **nrm;
  
  long sub_hed[4480]; 
  int fr,i,j;
  long matrix,offset;
  short sh_arr[6274];
  int **blk1=NULL, **blk2=NULL;
  double dtime_fct[4];
 

  /* Test byte-order of hardware architecture 0=PC, 1=HP/SUN */
  BYTE_ORDER=test_byte_order();

 

 fp=fopen(fname,"r");
 if (fp==NULL)
   {
     fprintf(stderr,"Can't read normalization-file %s\n\n",fname);
     exit(-1);
   }
 
 img_code = get_idl_file_info(&mh,sub_hed,fp,BYTE_ORDER);
 
 /* read frame */
 fr=0;
 
 /* get matrix number for this frame */
 matrix=img_code[plane][0][fr][0];
 /* set fp to beginning of sub_header */
 offset=(sub_hed[matrix]-1)*512;
 stat=fseek(fp,offset,SEEK_SET); /* point to sinogram subheader */
 if (stat)
   { 
     fprintf(stderr,"Unsuccesful fseek.\n\a\n");
     exit(-1);
   }
 /* read frame sub_header */
 read_subheader(&sh,fp,BYTE_ORDER);
 /*print_subheader(&sh);*/
 
 if (ftell(fp)-offset != 512)
   {fprintf(stderr,"Read error.\n\a\n");exit(-1);}
 
 /* Read frame */
 read_sh_arr(sh_arr, 6272, fp, BYTE_ORDER);
 
 
 nrm=(double**)multialloc(sizeof(double),2,56,112);
 
 /* write into counts of sino, here we append new frames as new angles */
 for (i=0;i<56;i++)
   for (j=0;j<112;j++)
     {
       nrm[i][j] = (double)sh_arr[j+112*i]; 
       
     }
 
 fclose(fp);
 multifree((void*)img_code,4);

 if (DEADTIME_FLAG)
   { 
     blk1=(int**)multialloc(sizeof(int),2,112,56);
     blk2=(int**)multialloc(sizeof(int),2,112,56);

     calc_deadtime_blocks(blk1, blk2);
     for (i=0;i<4;i++)
       dtime_fct[i] = (double)sh.cor_singles[i] / (double)sh.uncor_singles[i];
     
     for (i=0;i<56;i++)
       for (j=0;j<112;j++)
	 nrm[i][j] *= (dtime_fct[blk1[j][i]] * dtime_fct[blk2[j][i]]);
     
     multifree((void*)blk1,2);
     multifree((void*)blk2,2);

   }
       
       
       
 /*fp=fopen("nrm","w");
   for (i=0;i<56;i++)
   {
   for (j=0;j<112;j++)
   fprintf(fp,"%g ",nrm[i][j]);
   fprintf(fp,"\n");
   } 
   fclose(fp);
 */
       
  
 return(nrm);

}


/* This is a specialized top-level routine to 
   interface with the Purdue tomography reconstruction code.
   Routine reads sinogram for a single plane, i.e. 
   containing all frames for a single plane.
   Input : filename (cti scn file), plane to retrieve
   Output: double array of size 56*nframes by 112 containing sinogram, also
           sino->sep_d, sino->no_frames, sino->n_t , sino->n_theta ,
           sino->theta_0 , sino->halflife, sino->frames_starttime
   
   Routine is to be modified in order 
   to read multiple planes, but is small modification.
*/ 

/*
double** idl_read_plane_obsolete(char *fname, int plane, struct sino_info *sino) 
{
  FILE *fp;
  struct main_head mh;
  struct scn_head sh;
  long ****img_code; 
  int BYTE_ORDER,stat;

  long sub_hed[4480]; 
  int fr,i,j;
  long matrix,offset;
  short sh_arr[6274];
  double **Y;
  int **blk1,**blk2; 

  BYTE_ORDER=test_byte_order();

 

 fp=fopen(fname,"r");
 if (fp==NULL)
   {
     fprintf(stderr,"Can't read file %s\n\a\n",fname);
     exit(-1);
   }


 img_code = get_idl_file_info(&mh,sub_hed,fp,BYTE_ORDER);


 sino->no_frames = (int)mh.num_frames;
 sino->n_t = 112;
 sino->n_theta = 56*(int)mh.num_frames;
 sino->theta_0 = (double)mh.gantry_tilt;
 sino->sep_d=(double)mh.bed_elevation;
 sino->halflife=(double)mh.isotope_halflife;

 sino->frames_starttime=mget_spc(sino->no_frames,sizeof(int)); 
 sino->frames_duration=mget_spc(sino->no_frames,sizeof(int)); 
 blk1=(int**)multialloc(sizeof(int),2,112,56);
 blk2=(int**)multialloc(sizeof(int),2,112,56);

 calc_deadtime_blocks(blk1, blk2);

 fflush(stdout);fflush(stderr);

 printf("allocating sinogram...");fflush(stdout);fflush(stderr);
 Y=(double**)multialloc(sizeof(double),2,56*sino->no_frames,112);
 sino->deadtime_cor=(double**)multialloc(sizeof(double),2,56*sino->no_frames,112);
 printf("done.\n");fflush(stdout);fflush(stderr);

 for (fr=0;fr<sino->no_frames;fr++)  
   for (i=0;i<56;i++)
     for (j=0;j<112;j++)
       sino->deadtime_cor[fr*56+i][j] = 0.0;


 for (fr=0;fr<sino->no_frames;fr++)
   {
     matrix=img_code[plane][0][fr][0];

     offset=(sub_hed[matrix]-1)*512;
     stat=fseek(fp,offset,SEEK_SET);
     if (stat)
       { 
        fprintf(stderr,"Unsuccesful fseek.\n\a\n");
        exit(-1);
       }
     read_subheader(&sh,fp,BYTE_ORDER);
     sino->frames_starttime[fr]=(int)sh.frame_start_time/1000.0;
     sino->frames_duration[fr]=(int)sh.frame_duration/1000.0;
 
     if (ftell(fp)-offset != 512)
       {fprintf(stderr,"Read error.\n\a\n");exit(-1);}
     
     read_sh_arr(sh_arr, 6272, fp, BYTE_ORDER);
     
     for (i=0;i<56;i++)
       for (j=0;j<112;j++)
         {
           Y[fr*56+i][j] = (double)sh_arr[j+112*i]; 
	 }
     
     for (i=0;i<56;i++)
       for (j=0;j<112;j++)
	 sino->deadtime_cor[56*fr+i][j]=(double)sh.uncor_singles[blk1[j][i]]/(double)sh.cor_singles[blk1[j][i]] *(double)sh.uncor_singles[blk2[j][i]]/(double)sh.cor_singles[blk2[j][i]];
   }

 fclose(fp);


 multifree((void*)img_code,4);




 multifree((void*)blk1,2);
 multifree((void*)blk2,2);

 return(Y);



}
*/



/* plane here from 0-14 */
double** cti_read_attn_sin(char *fname, int plane, int *N_ang, int *M_offsets) 
{
  FILE *fp;
  struct main_head mh;
  struct scn_head sh;
  long ****img_code; /* img_code[plane][gate][frame][bed] */
  int BYTE_ORDER,stat;

  long sub_hed[4480]; 
  int i,j;
  long matrix,offset;
  short *sh_arr;
  double **Y;
  double scale_fct;
  int N,M;

  /* Test byte-order of hardware architecture 0=PC, 1=HP/SUN */
  BYTE_ORDER=test_byte_order();

 

 fp=fopen(fname,"r");
 if (fp==NULL)
   {
     fprintf(stderr,"cti_read_attn_sin: can't read file %s\n\a\n",fname);
     exit(-1);
   }


 img_code = get_idl_file_info(&mh,sub_hed,fp,BYTE_ORDER);
 matrix=img_code[plane][0][0][0];
 fflush(stdout);fflush(stderr);
 /* set fp to beginning of sub_header */
 offset=(sub_hed[matrix]-1)*512;
 stat=fseek(fp,offset,SEEK_SET); /* point to sinogram subheader */
 if (stat)
   { 
     fprintf(stderr,"Unsuccesful fseek.\n\a\n");
     exit(-1);
   }
 /* read frame sub_header */
 read_subheader(&sh,fp,BYTE_ORDER);
 print_subheader(&sh);
 scale_fct=(double)sh.scale_factor;
 
 M=(int)sh.dimension_1;
 N=(int)sh.dimension_2;


 Y=(double**)multialloc(sizeof(double),2,N,M);
 sh_arr=(short*)get_spc(N*M,sizeof(short));

 if (ftell(fp)-offset != 512)
   {fprintf(stderr,"Read error.\n\a\n");exit(-1);}
     
 /* Read frame */
 
 read_sh_arr(sh_arr, N*M, fp, BYTE_ORDER);
     
 /* write into counts of sino, here we append new frames as new angles */
 for (i=0;i<N;i++)
   for (j=0;j<M;j++)
     {
       Y[i][j] = scale_fct*(double)sh_arr[j+M*i]; 
     }
     
 fclose(fp);
 
 /*fp=fopen("attn","w");
 for (i=0;i<N;i++)
   { 
     for (j=0;j<M;j++)
       fprintf(fp,"%g ",Y[i][j]);
     fprintf(fp,"\n");
   }  
 fclose(fp);   */
 
 free((void*)sh_arr);
 multifree((void*)img_code,4);
 

 *N_ang=N;
 *M_offsets=M;

 return(Y);

}


/* Routine reads main header as well as all directories 
   of a CTI scn file. Returns array of matrix numbers in file
   indexed over planes/frames/gates/bed_pos. 
   Inputs : file-pointer to opened file, byte-order of current hardware
            allocated array long *sub_hed
   Returns: matrix-number=img_code[plane][gate][frame][bed_pos] 
            long *sub_hed containing the pointers to the first data 
	    byte for each matrix. 
*/
long**** get_idl_file_info(struct main_head *mh, long *sub_hed, FILE *fp, int BYTE_ORDER)
{
 long dirbuff[128];
 long offset,dir_block_ptr;
 int stat,i;
 long nimg,n_matrix=0,temp;
 long img_id[4480],last_rec[4480];
 long ****img_code;
 long nframe,ngt_bd,nbed,nplane,ngate;
 /*long trues=0; 
 struct scn_head sh;*/


 read_main_header(mh,fp,BYTE_ORDER);
 /*print_main_head(mh);*/
 
 /* allocate img_code_array */
 if (mh->num_planes < 1)
   mh->num_planes = 64;
 if (mh->num_gates < 1)
   mh->num_planes = 1;
 if (mh->num_frames<1)
   mh->num_frames=1;
 if (mh->num_bed_pos<1)
   mh->num_bed_pos=1;
 img_code=(long****)multialloc(sizeof(long),4,mh->num_planes,mh->num_gates,mh->num_frames,mh->num_bed_pos);
 
 
 
 
 /* put dir_pointer on second block, i.e. on first directory */
 dir_block_ptr = (long)2;
 
 /* read all_directories */
 nimg=0;
 for (;;)
   {
     offset=(long)((dir_block_ptr-1)*512);
     stat=fseek(fp,offset,SEEK_SET);
     if (stat)
       {
	 fprintf(stderr,"Unsuccesful fseek.\n\a\n");
	 exit(-1);
       }
     stat=fread(dirbuff,sizeof(long),128,fp);
     if (stat!=128)
       {
	 fprintf(stderr,"Read wrong number of bytes for dir.\n\a\n");
	 exit(-1);
       }
       
     /*printf("read block for dir_block_ptr=%d\n",dir_block_ptr);*/
 
     if (BYTE_ORDER)
       {
        dir_block_ptr = Long_Rev(dirbuff[1]);
        n_matrix = Long_Rev(dirbuff[3]);
        for (i=1;i<=n_matrix;i++)
          {
            img_id[nimg+i-1]   = Long_Rev(dirbuff[4*(i-1)+4]);
            sub_hed[nimg+i-1]  = Long_Rev(dirbuff[4*(i-1)+5]);
            last_rec[nimg+i-1] = Long_Rev(dirbuff[4*(i-1)+6]);
           }
       }    
     else
       {
        dir_block_ptr = dirbuff[1];
        n_matrix = dirbuff[3];
        for (i=1;i<=n_matrix;i++)
          {
            img_id[nimg+i-1]   = dirbuff[4*(i-1)+4];
            sub_hed[nimg+i-1]  = dirbuff[4*(i-1)+5];
            last_rec[nimg+i-1] = dirbuff[4*(i-1)+6];
          }
       } 
       
     temp = nimg;
     nimg = temp + n_matrix;
     
     /*printf("words=%d %d %d %d\n\n",Long_Rev(dirbuff[0]),Long_Rev(dirbuff[1]),Long_Rev(dirbuff[2]),Long_Rev(dirbuff[3]));
     */
     /* If this is last directory block, then pointer to 'next'
        dir points to first directory block 2 */
     if (dir_block_ptr ==2)
       break;
   } /* end for (;;) */
   
 /*printf("nimg=%d\n",(int)nimg);*/

  /* Set up image code array */
  for (i=1;i<=nimg;i++)
    {
      nframe=(long)(img_id[i-1] & (long)255);
      ngt_bd=(long)((img_id[i-1] & (long)65280)/(long)256);
      nbed = (long)((ngt_bd & (long)240)/(long)16);
      if (nbed<0)
        nbed=0;
      nplane = (long)((img_id[i-1] & (long)16711680)/(long)65536);   
      ngate = (long)((img_id[i-1] & (long)2130706432)/(long)16777216);
      if ((nplane>mh->num_planes) || (ngate>mh->num_gates) || (nframe>mh->num_frames) || (nbed>mh->num_bed_pos))
        {
          fprintf(stderr,"Error: number of planes/gates/frames/bed_pos in header doesn't agree with matrix labels in directory entries.\n\a\n");
          exit(-1);
         }

      /*printf("nplane-1=%d ngate-1=%d nframe-1=%d nbed=%d img_code=%d\n",nplane-1,ngate-1,nframe-1,nbed,i-1);*/
      img_code[nplane-1][ngate-1][nframe-1][nbed] = i-1;
     }
 
  /* trues=(long)0;
   for (i=0;i<nimg;i++)
     {
      offset=(sub_hed[i]-1)*512;
      stat=fseek(fp,offset,SEEK_SET); 
      if (stat)
        { 
	  fprintf(stderr,"Unsuccesful fseek.\n\a\n");
	  exit(-1);
        }
      
      read_subheader(&sh,fp,BYTE_ORDER);
      print_subheader(&sh);

      trues = trues + sh.net_trues;
     }
  */     

 return(img_code);

}




/* **************************************************************/
/********   ROUTINES FOR WRITING CTI IMAGE FILE ***************/
/***************************************************************/

/* Top level routine to interface Purdue tomography 
   reconstruction code. Right now, this routine 
   creates a new cti-img  file and writes a single image AS 2 PLANES to it.
  (The IUPUI IDL image display routine  /usr6/pet/idl/activation/rev_2.bat 
   will only work with >=2 planes since it interpolates the volume between planes).

   This routine can easily be modified to write multiple different planes, 
   i.e. write header first and then write multiple images s
   imilar as in osem_main.pro 
   INPUTS: filename (CTI .img file to be created)
           img as double array (data to be written )
	   N,M dimension of image (y,x respectively, only tested for N=M)
	   struct sino and geom 
   RETURNS: nothing. (Exits or error).
*/
void write_cti_image_file(char *fname, double **img, int N, int M, struct sino_info *sino, struct geom_info *geom)
{
  struct main_head mh;
  struct img_shead ish;
  double mx,quant;
  int sz_img_arr,i,j;
  FILE *fp;
  short *sh_img_arr; /* one dimensional array that will conating img in fix-pt format */
  int BYTE_ORDER;
  short img_min, img_max;

  BYTE_ORDER=test_byte_order();

  /* before doing anything, (see if we can) open file */
  fp=fopen(fname,"wb+"); /* create binary file for read/write */

  if (fp==NULL)
    {
      fflush(stdout);fflush(stderr);
      fprintf(stderr,"Error in write_cti_image_file: Can't open file %s for writing.\n\a\n",fname);
      exit(-1);
    }


  sz_img_arr=N*M;
  sh_img_arr=(short*)mget_spc(sz_img_arr,sizeof(short));

  /* initialize headers with zero */
  init_main_header(&mh);
  init_img_head(&ish);

  /* Convert double img to fix point short format */
  /* 1. find max */
  mx=fabs(img[0][0]);
  for (i=0;i<N;i++)
    for (j=0;j<M;j++)
      if (mx<fabs(img[i][j]))
	mx=fabs(img[i][j]);

 
  quant = mx/32767.0;
  
  for (i=0;i<N;i++)
    for (j=0;j<M;j++)
      sh_img_arr[j+M*i] = (short)(img[i][j]/quant);
 
  img_max=img_min=sh_img_arr[0];
  for (i=0;i<sz_img_arr;i++)
    {
      if (img_max < sh_img_arr[i])
	img_max = sh_img_arr[i];
      if (img_min > sh_img_arr[i])
	img_min = sh_img_arr[i];
    }
	  

 
  /* Write Main header */
  /* need to set some variables in main_header here: */
  mh.num_planes=sino->no_planes;
  mh.num_frames=1;
  mh.num_gates=1;
  mh.num_bed_pos=0;
  mh.init_bed_pos=0.0;
  mh.plane_separation = 0.63;
  mh.gantry_tilt=0.0;
  mh.gantry_rotation=0.0;
  mh.bed_elevation=0.0;
  mh.axial_fov=(float)geom->field_of_view;
  mh.transaxial_fov=sino->sep_d;
  mh.data_type=2;
  mh.file_type=2;

  write_main_header(fp, &mh, BYTE_ORDER);

  /* Set values in image subheader */
  ish.data_type=2;
  ish.num_dimensions=2;
  ish.dimension_1=(short)N;
  ish.dimension_2=(short)M;
  /*printf("N=%d M=%d\n",N,M);*/
  ish.recon_scale=1.0;
  ish.slice_width=mh.plane_separation;
  ish.frame_duration= (long)sino->frame_duration; 
  ish.frame_start_time= (long)sino->frame_start_time; 
  ish.plane_eff_corr_fctr=1.0;
  ish.decay_corr_fctr=1.0;
  ish.loss_corr_fctr=1.0;
  ish.quant_units=2;
  ish.ecat_calibration_fctr=1.0;
  ish.well_counter_cal_fctr = 1.0;

  ish.quant_scale=(float)quant;
  ish.pixel_size=geom->field_of_view/(float)N;
  ish.image_max = img_max;
  ish.image_min = img_min;


  /*for (p=1;p<=mh.num_planes;p++)
    write_cti_img_matrix(fp,&ish, p,1,1,0, sh_img_arr,sz_img_arr,BYTE_ORDER);*/
  write_cti_img_matrix(fp,&ish, 1,1,1,0, sh_img_arr,sz_img_arr,BYTE_ORDER);

  

  
  free((void*)sh_img_arr);
  fclose(fp);


}




void add_cti_image_mat(char *fname, double **img, int N, int M, struct sino_info *sino, struct geom_info *geom,int ct_pl)
{
  struct main_head mh;
  struct img_shead ish;
  double mx,quant;
  int sz_img_arr,i,j;
  FILE *fp;
  short *sh_img_arr; /* one dimensional array that will conating img in fix-pt format */
  int BYTE_ORDER;
  short img_min, img_max;

  BYTE_ORDER=test_byte_order();

  /* before doing anything, (see if we can) open file */
  fp=fopen(fname,"rb+"); /* create binary file for read/write */

  if (fp==NULL)
    {
      fflush(stdout);fflush(stderr);
      fprintf(stderr,"Error in write_cti_image_file: Can't open file %s for writing.\n\a\n",fname);
      exit(-1);
    }

  sz_img_arr=N*M;
  sh_img_arr=(short*)mget_spc(sz_img_arr,sizeof(short));

  /* initialize headers with zero */
  init_main_header(&mh);
  init_img_head(&ish);

  /* Convert double img to fix point short format */
  /* 1. find max */
  mx=fabs(img[0][0]);
  for (i=0;i<N;i++)
    for (j=0;j<M;j++)
      if (mx<fabs(img[i][j]))
	mx=fabs(img[i][j]);

 
  quant = mx/32767.0;
  
  for (i=0;i<N;i++)
    for (j=0;j<M;j++)
      sh_img_arr[j+M*i] = (short)(img[i][j]/quant);
 
  img_max=img_min=sh_img_arr[0];
  for (i=0;i<sz_img_arr;i++)
    {
      if (img_max < sh_img_arr[i])
	img_max = sh_img_arr[i];
      if (img_min > sh_img_arr[i])
	img_min = sh_img_arr[i];
    }
	  

 
 

  /* Set values in image subheader */
  ish.data_type=2;
  ish.num_dimensions=2;
  ish.dimension_1=(short)N;
  ish.dimension_2=(short)M;
  /*printf("N=%d M=%d\n",N,M);*/
  ish.recon_scale=1.0;
  ish.slice_width=mh.plane_separation;
  ish.frame_duration= (long)sino->frame_duration; 
  ish.frame_start_time= (long)sino->frame_start_time; 
  ish.plane_eff_corr_fctr=1.0;
  ish.decay_corr_fctr=1.0;
  ish.loss_corr_fctr=1.0;
  ish.quant_units=2;
  ish.ecat_calibration_fctr=1.0;
  ish.well_counter_cal_fctr = 1.0;

  ish.quant_scale=(float)quant;
  ish.pixel_size=geom->field_of_view/(float)N;
  ish.image_max = img_max;
  ish.image_min = img_min;


  write_cti_img_matrix(fp,&ish, ct_pl+1,1,1,0, sh_img_arr,sz_img_arr,BYTE_ORDER);

  

  
  free((void*)sh_img_arr);

  fclose(fp);

}


/* This routine adds a matrix to an existing cti file. 
   Before calling this, main header must be written. */
void write_cti_img_matrix(FILE *fp, struct img_shead *ish, long plane, long frame, long gate, long bed, short *sh_img_arr, int sz_img_arr, int BYTE_ORDER)
{
  long index;
  int stat;
  
  index = Update_dir(fp, plane,frame, gate, bed, sz_img_arr, BYTE_ORDER);

  stat=fseek(fp,index,SEEK_SET);
  if (stat)
    { 
      fprintf(stderr,"Unsuccesful fseek in write_cli_plane [1].\n\a\n");
      exit(-1);
    }

  /* write subheader */
  /*write_subheader(sh,fp,BYTE_ORDER);*/
  write_img_shead(fp,ish,BYTE_ORDER);
  
  write_sh_arr(sh_img_arr, sz_img_arr, fp, BYTE_ORDER);
}




/* Subroutine for write_cti_img_matrix: Before adding the 
   new matrix, we have to find the spot to where it will 
   be written (return value is a pointer to where matrix 
   data starts). Also, this routine updates the directory 
   entry to contain the new matrix. If the current directory block
   is full, it creates a new directory block. 
*/
long Update_dir(FILE *fp,long plane, long frame, long gate, long bed, int sz_img_arr, int BYTE_ORDER)
{
 long dir_1st[128];
 long last_rec[32];
 long for_ptr=2,tmp_ptr,num_empty,num_stored,max_last_rec,new_dir_ptr=0,mat_lst_rec;
 long index,nxt_subh,bak_ptr,no_bytes_img,no_records_img,remainder;/*,mat_id;*/
 long i,mat_stat;
 int stat;
 unsigned char code[6];
 
 /* Find last directory record */
 for (;;)
   {
    index=(for_ptr-1)*512;
    stat=fseek(fp,index,SEEK_SET); 
    if (stat)
      {
       fprintf(stderr,"Unsuccesful fseek in Update_dir.\n\a\n");
       exit(-1);
      }
    read_lg_arr(dir_1st, 128, fp, BYTE_ORDER);

    tmp_ptr=for_ptr;
    num_empty=dir_1st[0];
    for_ptr=dir_1st[1];
    bak_ptr=dir_1st[2];
    num_stored=dir_1st[3];

    /* Only if forward prt points to first dir record (==2), then
       this directory is last one. Otherwise go on in loop until last record */
    if (for_ptr==2)
      break;
   }   

  /* Store location of last record of each matrix file into array called last_rec: */
  max_last_rec=0;
  for (i=0;i<num_stored;i++)
    {
     last_rec[i] = dir_1st[4*i+6];
     if (max_last_rec<last_rec[i])
       max_last_rec=last_rec[i];
    }
    
    
  /* See if current directory is full, if so, create new directory */
  if (num_stored==31)
    {
      new_dir_ptr = max_last_rec+1;
      
      /* now update 1st dir entry of last dir block to point here */
      for_ptr=new_dir_ptr;     
      
      /* go back to beginning of last dir block */
      stat=fseek(fp,index,SEEK_SET); 
      if (stat)
       {
        fprintf(stderr,"Unsuccesful fseek in Update_dir. [2]\n\a\n");
        exit(-1);
       }
      
      write_lg_arr(&num_empty, 1, fp, BYTE_ORDER);
      write_lg_arr(&for_ptr, 1, fp, BYTE_ORDER);
      write_lg_arr(&bak_ptr, 1, fp, BYTE_ORDER);
      write_lg_arr(&num_stored, 1, fp, BYTE_ORDER);

      /* update 1st index of newly created directory */
      index=512*(new_dir_ptr-1);
      stat=fseek(fp,index,SEEK_SET); 
      if (stat)
       {
        fprintf(stderr,"Unsuccesful fseek in Update_dir. [3]\n\a\n");
        exit(-1);
       }

      for_ptr=2;
      bak_ptr=tmp_ptr; /* new_dir_ptr-1 */
      num_stored=1;
      num_empty=30;
      
      write_lg_arr(&num_empty, 1, fp, BYTE_ORDER); 
      write_lg_arr(&for_ptr, 1, fp, BYTE_ORDER);
      write_lg_arr(&bak_ptr, 1, fp, BYTE_ORDER);
      write_lg_arr(&num_stored, 1, fp, BYTE_ORDER);
     }
   else
     {
      /* New matrix still fits in current directory block */
      /* update only number of matrix file stored and matrix file space available
         in current directory */
      
      stat=fseek(fp,index,SEEK_SET); 
      if (stat)
       {
        fprintf(stderr,"Unsuccesful fseek in Update_dir. [4]\n\a\n");
        exit(-1);
       }
      num_stored++;
      num_empty--;
      write_lg_arr(&num_empty, 1, fp, BYTE_ORDER); 
      write_lg_arr(&for_ptr, 1, fp, BYTE_ORDER);
      write_lg_arr(&bak_ptr, 1, fp, BYTE_ORDER);
      write_lg_arr(&num_stored, 1, fp, BYTE_ORDER);
     }

   /* Locate subheader record number new (image) matrix file */
   if (bak_ptr !=0)
     {
      if (num_stored == 1)
        nxt_subh=new_dir_ptr+1; /* only the case when created new dir, new_dir_ptr is initialized */ 
      else
        nxt_subh=max_last_rec+1;
     }
   else 
     {
      if (num_stored == 1)
        nxt_subh=3;
      else
        nxt_subh=max_last_rec+1;
     }
     
   mat_stat=1;


   /* determine last record of matrix */
   
   no_bytes_img=(long)(2*sz_img_arr);
   no_records_img=no_bytes_img/512;
   remainder= no_bytes_img%512;

   if (remainder!=0)
     no_records_img++;

   mat_lst_rec=nxt_subh + no_records_img;
   
   /* generate matrix id as 32bit long integer */
   /*
     code[0]=(unsigned char)gate;
     code[1]=(unsigned char)plane;
     code[2]=(unsigned char)(16*bed);
     code[3]=(unsigned char)frame;
   */
   code[3]=(unsigned char)gate;
   code[2]=(unsigned char)plane;
   code[1]=(unsigned char)(16*bed);
   code[0]=(unsigned char)frame;


 
   
   /* index is currently pointing at beginning of directory */
   /* so increase to point to dir-entry for new matrix */
   index=index+16*num_stored;
   stat=fseek(fp,index,SEEK_SET); 
   if (stat)
     {
      fprintf(stderr,"Unsuccesful fseek in Update_dir. [5]\n\a\n");
      exit(-1);
     }
      
   /* write dir entry for this matrix */
   /*memcpy(&mat_id, code, 4); This only works on HP byte order. Fixed below by writing bytes and reversing code manually above. TF, Nov 24, 99. 
     write_lg_arr(&mat_id, 1, fp, BYTE_ORDER); */
   stat=fwrite(code,sizeof(unsigned char),4,fp);
   if (stat!=4)
     {
      fprintf(stderr,"Error writing matrix id, stat=%d, len=%d.\n\a\n",stat,4);
      exit(-1);
    }
   write_lg_arr(&nxt_subh, 1, fp, BYTE_ORDER); 
   write_lg_arr(&mat_lst_rec, 1, fp, BYTE_ORDER); 
   write_lg_arr(&mat_stat, 1, fp, BYTE_ORDER); 

   /* point index at beginning of sub_header */
   index=(nxt_subh-1)*512;
   
   return(index);
}



double** cti_read_sino(char **fnames, int no_input_files, int plane, struct sino_info *sino)
{
  FILE *fp;
  struct main_head mh;
  struct scn_head sh;
  long ****img_code; /* img_code[plane][gate][frame][bed] */
  int BYTE_ORDER,stat;
 
  long sub_hed[4480]; 
  int fr,i,j;
  long matrix,offset;
  short sh_arr[6274];
  double **Y=NULL;
  int im;
  

  /* Test byte-order of hardware architecture 0=PC, 1=HP/SUN */
  BYTE_ORDER=test_byte_order();

  sino->dtime.scan_start_second = (int*)get_spc(no_input_files,sizeof(int));
  sino->dtime.scan_start_minute = (int*)get_spc(no_input_files,sizeof(int));
  sino->dtime.scan_start_hour = (int*)get_spc(no_input_files,sizeof(int));
  sino->dtime.scan_start_day = (int*)get_spc(no_input_files,sizeof(int));
  


  /* loop over scn files to be read and added onto Y */
  for (im=0;im<no_input_files;im++)
    { 
      fp=fopen(fnames[im],"r");
      if (fp==NULL)
	{
	  fprintf(stderr,"Can't read file %s\n\a\n",fnames[im]);
	  exit(-1);
	}
      
      img_code = get_idl_file_info(&mh,sub_hed,fp,BYTE_ORDER);

      if (im==0)
	{
	  sino->no_frames = (int)mh.num_frames;
	  sino->n_t = 112;
	  sino->n_theta = 56*(int)mh.num_frames;
	  sino->theta_0 = (double)mh.gantry_tilt;
	  sino->sep_d=(double)mh.bed_elevation;
	  sino->halflife=(double)mh.isotope_halflife;
  
	  sino->frames_starttime=(int**)multialloc(sizeof(int),2,no_input_files,sino->no_frames); 
	  sino->frames_duration =(int**)multialloc(sizeof(int),2,no_input_files,sino->no_frames);
	  sino->dtime.cor_singles = (double***)multialloc(sizeof(double),3,no_input_files,sino->no_frames,4);
	  sino->dtime.uncor_singles = (double***)multialloc(sizeof(double),3,no_input_files,sino->no_frames,4);
	   
	  fflush(stdout);fflush(stderr);
  
	  printf("allocating sinogram...");fflush(stdout);fflush(stderr);
	  Y=(double**)multialloc(sizeof(double),2,56*sino->no_frames,112);
	  
	  printf("done.\n");fflush(stdout);fflush(stderr);
  
	 
	  /* Initialize Y to zero */
	  for (fr=0;fr<sino->no_frames;fr++)
	    for (i=0;i<56;i++)
	      for (j=0;j<112;j++)
		Y[56*fr+i][j]=0.0;
	}
      else /* im > 0 */
	{
	  /* check if new file has same no_frames etc. */
	  check_addtl_file(&mh, sino, im);
	}

      sino->dtime.scan_start_second[im] = (int)mh.scan_start_second;
      sino->dtime.scan_start_minute[im] = (int)mh.scan_start_minute;
      sino->dtime.scan_start_hour[im] = (int)mh.scan_start_hour;
      sino->dtime.scan_start_day[im] = (int)mh.scan_start_day;
      
      /* read frames */
      for (fr=0;fr<sino->no_frames;fr++)
	{
	  /* get matrix number for this frame */
	  matrix=img_code[plane][0][fr][0];
      
	  /* set fp to beginning of sub_header */
	  offset=(sub_hed[matrix]-1)*512;
	  stat=fseek(fp,offset,SEEK_SET); /* point to sinogram subheader */
	  if (stat)
	    { 
	      fprintf(stderr,"Unsuccesful fseek.\n\a\n");
	      exit(-1);
	    }
	  /* read frame sub_header */
	  read_subheader(&sh,fp,BYTE_ORDER);
	  /*print_subheader(&sh);*/
	  sino->frames_starttime[im][fr]=(int)sh.frame_start_time/1000.0; /* from msec to sec */
	  sino->frames_duration[im][fr]=(int)sh.frame_duration/1000.0; /* from msec to sec */
      
	  for (i=0;i<4;i++)
	    {
	      sino->dtime.cor_singles[im][fr][i]=(double)sh.cor_singles[i];
	      sino->dtime.uncor_singles[im][fr][i]=(double)sh.uncor_singles[i];
	    }

	  if (ftell(fp)-offset != 512)
	    {fprintf(stderr,"Read error.\n\a\n");exit(-1);}
     
	  /* Read frame */
	  read_sh_arr(sh_arr, 6272, fp, BYTE_ORDER);
     
	  /* write into counts of sinogram, here we append 
	     new frames as new angles */
	  for (i=0;i<56;i++)
	    for (j=0;j<112;j++)
	      Y[fr*56+i][j] += (double)sh_arr[j+112*i]; 
	}
      
      fclose(fp);
      multifree((void*)img_code,4);
    } /* end for im */
  
  
  /*fp=fopen("Y","w");
    for (i=0;i<56*sino->no_frames;i++)
    {
    for (j=0;j<112;j++)
    fprintf(fp,"%g ",Y[i][j]);
    fprintf(fp,"\n");
    } 
    fclose(fp);
  */ 

  return(Y);

}



void check_addtl_file(struct main_head *mh, struct sino_info *sino, int im)
{
  int stat=0;

  if (sino->no_frames != (int)mh->num_frames)
    {
      fprintf(stderr,"Number of frames in different input files. First file has %d frames, current file %d has %d frames\n\a\n",sino->no_frames,im,(int)mh->num_frames);
      stat=1;
    }

  if (fabs(sino->theta_0 - (double)mh->gantry_tilt)>0.000001)
    {
      fprintf(stderr,"theta_0/gantry_tilt different in different input files. First file: %f, current file(%d): %f\n\a\n",sino->theta_0,im, (double)mh->gantry_tilt);
      stat=1;
    }
  
  if (fabs(sino->sep_d - (double)mh->bed_elevation) > 0.000001)
    {
      fprintf(stderr,"sep_d/.bed_elevation different in different input files. First file: %f, current file(%d): %f\n\a\n",sino->sep_d,im, (double)mh->bed_elevation);
      stat=1;
    }

  
  if (fabs(sino->halflife - (double)mh->isotope_halflife) > 0.000001)
    {
      fprintf(stderr,"halflife different in different input files. First file: %f, current file(%d): %f\n\a\n",sino->halflife,im, (double)mh->isotope_halflife);
      stat=1;
    }
  

  if (stat)
    exit(-1);

}
 





