#include <stdio.h>
#include <math.h>
#include <string.h>
#include "ansi_lib_loc/io_mat_dbl.h"
#include "ansi_lib_loc/cti_io.h"
#include "ansi_lib_loc/allocate.h"
#include "ansi_lib_loc/A_comp_iupui.h"
#include "fr_icm.h"

#ifdef __linux__
#include <getopt.h>
#endif
extern int optind;
extern char *optarg;


struct cmdl{
  char input[256];
  char output[256];
  char nrmfile[256];
  char sysres_file[256];
  int plane;
  int NRM_FLAG;
};


void cmdline_error(void);
void parse_cmdline(struct cmdl *cmdline,int argc,char **argv);
void y_mask(struct all_data *data);
void write_output(struct cmdl *cmdline,struct all_data *data, struct system_resp *sysres);


void main( int argc, char **argv )
{
  int pr,i,ind,pind;
  struct cmdl cmdline;
  struct all_data data;
  double nrm;
  struct system_resp sysres;
  char **fname;

  
  /* parse command-line */
  parse_cmdline(&cmdline,argc,argv);

  /* read input */
  fname=(char**)multialloc(sizeof(char),2,1,256);
  strcpy(fname[0],cmdline.input);
 
  data.Y=cti_read_sino(fname, 1,cmdline.plane, &data.sino); 

  multifree((void*)fname,2);
  sysres.sep_d=data.sino.sep_d;

  printf("theta_0=%g sep_d=%g no_frames=%d\n\n",data.sino.theta_0,data.sino.sep_d,data.sino.no_frames);

  /* if -n flag, read nrm array, apply cos-correction, divide sinogram*/
  if (cmdline.NRM_FLAG)
    {
      data.sino.nrm_img=read_nrm_plane(cmdline.nrmfile, cmdline.plane,1);
      
      /* calc sino_arrays, also does cos correction of nrm-array */
      data.sino.nrm_FLAG=1;
      data.sino.plane=cmdline.plane;
      data.sino.DECAY_FLAG=0;
      data.sino.attn.ATTN_MODE=0;
      read_system_response(cmdline.sysres_file,&sysres);
      sysres.sep_d = data.sino.sep_d;
      printf("Using sep_d=%f from sinogram (not response file).\n",data.sino.sep_d);
 
      calc_sino_arrays(&data.sino,&sysres);
      
      /* Calculate normalization correction */
      for(pr=0; pr < data.sino.n_theta; pr++)
	for(i = 0; i < data.sino.n_t; i++) 
	  {   
	    pind = pr*data.sino.n_t;
	    ind = pind+i; /* linear index in 1D array*/
	    
	    nrm=(double)data.sino.nrm_arr[ind];
	    if (nrm<1e-5)
	      nrm=60.0;

	    data.Y[pr][i]  /= nrm;
	  }
    }

  /* mask sinogram: set values outside diamond to zero */
  y_mask(&data);


 
 /* write_output */
  write_output(&cmdline,&data,&sysres);
  
  
  multifree((void*)data.Y,2);
  if (cmdline.NRM_FLAG)
    {
      multifree((void*)data.sino.nrm_img,2);
      free((void*)data.sino.nrm_arr);
    }
  /* Note: does not free all arrays generated in calc_sino_arr right now. */ 

}




void cmdline_error()
{
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"conv_sino [-n normalization] [-p plane, default=0]  sysres_file input output\n");
  fprintf(stderr,"\nNotice: sysres_file is only read when -n is specified.\n");
  exit(-1);  
}

void parse_cmdline(struct cmdl *cmdline,int argc,char **argv)
{
  int i,ct;
  int stat_plane=0;

  cmdline->NRM_FLAG=0;
  cmdline->plane=0;

  for (;;)
    { 
      i= getopt(argc, argv, "n:p:");
      if (i==EOF)
        break;
      else
        {
          if (i == '?')
            cmdline_error();
          if (i=='n')
            {
             cmdline->NRM_FLAG=1;
             strcpy(cmdline->nrmfile,optarg);  
            }
	  if (i=='p')
            {
	      cmdline->plane = atoi(optarg);
	      stat_plane=1;
            }
	 
	  
        }
    }

  ct=optind;
  
  if (argc-optind != 3)
    {
      fprintf(stderr,"\nError: Wrong number of arguments.\n\a\n");
      cmdline_error();
    }

  if (!stat_plane) 
    {
      fprintf(stderr,"Warning: no plane selected. Converting plane 0.\n\n");
    }
  
  printf("Converting plane %d.\n",cmdline->plane);

  
  strcpy(cmdline->sysres_file,argv[ct]);
  ct++;
 
  strcpy(cmdline->input,argv[ct]);
  ct++;
  
  strcpy(cmdline->output,argv[ct]);
  ct++;
}


  
void write_output(struct cmdl *cmdline,struct all_data *data, struct system_resp *sysres)
{
  MATFile *fout;
  char matrix[20];
  double tmp, *ptmp;

  printf("Writing output image %s ...\n",cmdline->output);
  
  /* open file */
  fout = matOpen(cmdline->output, "w");
  if (fout == NULL)
    {
      fprintf(stderr,"Cannot write MatLab file %s\n",cmdline->output);
      exit(-1);
    }
  
  /* write parameters */
  sprintf(matrix,"no_frames");
  tmp=(double)data->sino.no_frames;
  ptmp=&tmp;
  write_mat_double_image(&ptmp,1,1,matrix,fout);

  sprintf(matrix,"sep_d");
  tmp=sysres->sep_d;
  ptmp=&tmp;
  write_mat_double_image(&ptmp,1,1,matrix,fout);
 
  sprintf(matrix,"theta_0");
  tmp=data->sino.theta_0;
  ptmp=&tmp;
  write_mat_double_image(&ptmp,1,1,matrix,fout);

  
  sprintf(matrix,"counts");  
  write_mat_double_image(data->Y,data->sino.n_theta,data->sino.n_t,matrix,fout);
  
  matClose(fout);
}
  

 
void y_mask(struct all_data *data)
{
  double **Y;
  int fr,i,j,no_frames;
  int det1[112][56],det2[112][56];

  printf("Masking out sinogram values outside diamond.\n");

  no_frames=data->sino.no_frames;
  Y=data->Y;

  for (i=0;i<56;i++)
    for (j=0;j<112;j++)
      {
	det1[j][i] = ( (int)(84.0+(double)i+((double)j)/2.0) )%112; /* dbot*/
	det2[j][i] = ( (int)(84.0+(double)i-((double)j)/2.0) )%112; /* dtop*/

	if ((det1[j][i]>55)||(det2[j][i]<56))
	  for (fr=0;fr<no_frames;fr++)
	    Y[fr*56+i][j]=0.0;
	  
      }
}
