#include <stdio.h>
#include <string.h>
#include <math.h>

#include "fr_recon.h"
#include "fr_paramest.h"
#include "fr_icm.h"
#include "read_cfg_file.h"
#include "iu_utils.h"

#include "ansi_lib_loc/A_comp_iupui.h"
#include "ansi_lib_loc/allocate.h"
#include "ansi_lib_loc/assort.h"
#include "ansi_lib_loc/cti_io.h"
#include "ansi_lib_loc/io_mat_dbl.h"
#include "ansi_lib_loc/randlib.h"
#include "ansi_lib_loc/solve2.h"

#ifdef __linux__
#include <getopt.h>
#endif
extern int optind;
extern char *optarg;
int VERBOSE=1;


/*#define FUDGE_SIG 1.0*/
#define PI_OWN 3.14159265358979
#define CLIP_Y
/*#define WRITE_YCOR*/


void change_parameters_for_sgl_frame_recon(struct all_data *data, struct cmdline_params *cmdline);
void cmdline_error(void);
void parse_cmdline(struct cmdline_params *cmdline,int argc,char **argv);
void print_configuration(struct cfg_info *cfg_info, struct cmdline_params *cmdline, struct all_data *data);
void set_parameters(struct cmdline_params *cmdline,struct cfg_info *cfg_info,struct all_data *data,struct geom_info *geom);


void read_data(struct all_data *data,struct column *sp_col,struct cfg_info *cfg_info,int resol);
void write_output(struct cfg_info *cfg_info,struct all_data *data,double **sigma,struct geom_info *geom);


void init_sp_col(struct column *sp_col,struct sino_info *sino);
double recon(struct all_data *data,struct geom_info *geom,struct column *sp_col,struct cfg_info *cfg_info);


void free_memory(struct all_data *data, struct column *sp_col,double **sigma,struct cfg_info *cfg_info);




void main( int argc, char **argv )
{
  struct geom_info geom;
  struct column sp_col;
  struct cmdline_params cmdline;
  struct all_data data;
  double **sigma;
  struct cfg_info cfg_info;


  srandom2(1);



  parse_cmdline(&cmdline,argc,argv);

  printf("Reading configuration file %s...",cmdline.cfg_fname);fflush(stdout);
  read_cfg_file(cmdline.cfg_fname,&cfg_info);
  printf("done.\n\n");fflush(stderr);fflush(stdout);

  set_parameters(&cmdline,&cfg_info,&data,&geom);
  check_parameters(&cfg_info);

  printf("Reading files...\n");fflush(stdout);
  read_data(&data,&sp_col,&cfg_info,cfg_info.resol);
  printf("done reading files.\n\n");fflush(stderr);fflush(stdout);

  /* If single frame reconstruction specified in cmdline,
     set no_frames=1, copy frame sino data to the beg. of Y etc. */
  if (cmdline.single_frame!=-1)
    change_parameters_for_sgl_frame_recon(&data,&cmdline);
      
  if (cfg_info.verbose)
    print_cfg_info(&cfg_info);

  if (cfg_info.addposi_FLAG)
    y_positivity(&data);
  
#ifdef CLIP_Y
  clip_y(&data);
#endif
  
  /* set sinogram entries outside diamond to zero */
  y_mask(&data);
  
  /* Now eliminate activity outside prj_max_t in initial condition */
  truncate_init(&data, &geom);
  
  /* Allocate memory and initialize projection matrices*/
  init_sp_col(&sp_col,&data.sino);
  sigma = (double **)get_img(1,1,sizeof(double));
  
  init_A_matrix(&data.sino,&geom, cfg_info.sysres_fname);
  
  if (cfg_info.verbose)
    print_configuration(&cfg_info, &cmdline, &data);

  /* fix-res reconstruction */
  sigma[0][0] = recon(&data,&geom,&sp_col,&cfg_info);
  
  /* Write Matlab output */
  write_output(&cfg_info,&data,sigma,&geom);
  

  /* Write CTI-Format output */  
  /*sprintf(fname,"%s.img",cmdline.output);
    if (ct_pl==0)
    write_cti_image_file(fname, data.X, geom.n_y, geom.n_x, &data.sino, &geom);
    else
    add_cti_image_mat(fname, data.X, geom.n_y, geom.n_x, &data.sino, &geom,ct_pl);
  */
  free_A_matrix(&data.sino,&geom);
  free_memory(&data, &sp_col,sigma,&cfg_info);

  

  
}




double recon(struct all_data *data,struct geom_info *geom,struct column *sp_col,struct cfg_info *cfg_info)
{
  
  int i;/*,j;*/
  /*double smY,smS;*/

  double **profiles;  
  static double *S;
  static int param, N,M;
  static double SX=0.0,P;

  srandom2(1);
  P = cfg_info->p-1;
  profiles=NULL;

  N = geom->n_y;
  M = geom->n_x;
 
#ifdef WRITE_YCOR 
  printf("**************writing Y\n\n\n\n");
  write_Ycor(data); /* write nrm/decay corrected sino to disk */
#endif
  

  /* calculate number of pixels w/ nonzero mask  Nsup */
  data->Nsup=0;
  for(i=0;i<N*M;i++)
    if(data->mask[0][i])
      data->Nsup++;
  
  /* estimate parameter sigma */
  if( cfg_info->no_iter_param != 0)
    SX=parameter_estimation(data, geom,profiles, cfg_info,sp_col, P);
  else
    {
      /*SX=enter_parameter(data, geom, cmdline,P);*/
      SX=pow(cfg_info->sigma,cfg_info->p);
    }
  
  printf("Sigma is %f\n",pow(SX,1.0/cfg_info->p));

#ifdef FUDGE_SIG
  SX = SX * pow((double)FUDGE_SIG,(double)cfg_info->p);
  printf("FUDGE-factor of %f: Sigma is %f\n",(double)FUDGE_SIG,pow(SX,1.0/cfg_info->p));
#endif

  /* reconstruct */
  if(cfg_info->no_iter_rec != 0)
    {
      /* Allocate S  which is Ax*/
      S = (double *) get_spc(data->sino.n_t*data->sino.n_theta, sizeof(double));

      /* Calculate S */
      f_pro(S,data,geom,profiles,sp_col);



      /*smS=0;
	smY=0;
	for (i=0;i<data->sino.n_t*data->sino.n_theta;i++)
	{
	smS += (S[i]-data->scatter);
	}
	
	for (i=0;i<data->sino.n_theta;i++)
	for (j=0;j<data->sino.n_t;j++)
	smY += (data->Y[i][j]-data->scatter);
	
	printf("smS/smY=%g smS=%g smY=%g\n\n",smS/smY,smS,smY);
	
	

	exit(-1);
	
      */


      printf("Starting reconstruction ...\n");
      param = 0;
      SX=relax_recon(data,S, geom, profiles, cfg_info,sp_col,P,SX);

      
      printf("Finished reconstruction\n");
      free((void *)S);
    }
  
  
  return(pow(SX,1.0/(P+1)));
  
}


void init_sp_col(struct column *sp_col,struct sino_info *sino)
{
  sp_col->val = (double *) get_spc(sino->n_t*sino->n_theta, sizeof(double));
  sp_col->row = (int *) get_spc(sino->n_t*sino->n_theta, sizeof(int));
  sp_col->multiplier = 1.0; 
}


void cmdline_error()
{
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"emisnew [-p plane (default=0)] [-r recon_resolution] [-i no_iter] [-I (interpolate result)] [-s sigma] [-f singleframe_no [-t]] cfg_file init_file out_file\n\n");
  exit(-1);  
}



void parse_cmdline(struct cmdline_params *cmdline,int argc,char **argv)
{
  int i,ct;
  int stat_plane=0,stat_res=0;
 
  cmdline->plane=0;
  cmdline->single_frame=-1;
  cmdline->sglfr_th0cor_FLAG=0;
  cmdline->INTERP_FLAG=0;
  cmdline->SIGMA_FLAG=0;
  cmdline->no_iter_rec=-1;
  cmdline->resol=-1;

  for (;;)
    { 
      i= getopt(argc, argv, "p:r:i:Is:f:t");
      if (i==EOF)
        break;
      else
        {
          if (i == '?')
            cmdline_error();
	  if (i=='p')
            {
	      cmdline->plane = atoi(optarg);
	      stat_plane=1;
            }
	  if (i=='f')
            {
	      cmdline->single_frame = atoi(optarg);
	    }
	  if (i=='t')
            {
	      cmdline->sglfr_th0cor_FLAG = 1;
	    }
	  
	  if (i=='r')
	    {
	      cmdline->resol=atoi(optarg);
	      stat_res=1;
	    }
	  if (i=='s')
	    {
	      cmdline->sigma=atof(optarg);
	      cmdline->SIGMA_FLAG=1;
	    }
	  if (i=='I')
	    {
	      cmdline->INTERP_FLAG=1;
	    }
	  if (i=='i')
	    {
	      cmdline->no_iter_rec=atoi(optarg);
	    }
	  
	  
	}
    }
  ct=optind;
  if ((argc-optind < 3) || (argc-optind > 3))
    {
      fprintf(stderr,"\nError: Wrong number of arguments.\n\a\n");
      cmdline_error();
    }

  cmdline->cfg_fname = argv[ct];
  ct++;

  cmdline->initial = argv[ct];
  ct++;
  
  cmdline->output = argv[ct];
  ct++;

  if (!stat_res)
    printf("No resolution specified. Reconstructing at default resolution.\n");
  
  
  

}


void read_data(struct all_data *data,struct column *sp_col,struct cfg_info *cfg_info,int resol)
{
  MATFile *fin;  
  
  mxArray *pm;

  double **temp_image;
  int N,M,Ang,T,i,j;
  int Ang2,T2;
  char *fname;
  int sfile_plane;
  

  data->sino.nrm_FLAG=0;

  /* if set to matlab, read input sinogram and nrm file from matfiles */
  if (cfg_info->MATLAB_FLAG)
    {
      fname=cfg_info->sino_fname[0];
     /* read input sinogram Y */
     fin = matOpen(fname, "r");
     if (fin == NULL)
       {
         fprintf(stderr,"Cannot read MatLab input file %s\n",fname);
         exit(-1);
       }
  
     data->Y = read_mat_double_image(&Ang,&T,"counts",fin);
     if (data->Y == NULL)
       {
         fprintf(stderr,"Cannot read counts matrix \n");
         exit(-1);
       }
     /*read_mat_sino_info(fin, &data->sino);*/
 
    
      pm = matGetArray(fin,"sep_d");
      if(pm != NULL) 
	data->sino.sep_d = mxGetScalar(pm);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable sep_d.\n\a\n",fname);
	  exit(-1);
	}
      mxDestroyArray(pm);
      
      pm = matGetArray(fin,"theta_0");
      if(pm != NULL) 
	data->sino.theta_0 = mxGetScalar(pm);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable theta_0.\n\a\n",fname);
	  exit(-1);
	}
      mxDestroyArray(pm);
      pm = matGetArray(fin,"no_frames");
      if(pm != NULL) 
	data->sino.no_frames = (int)(mxGetScalar(pm)+0.5);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable no_frames.\n\a\n",fname);
	  exit(-1);
	}
      mxDestroyArray(pm);
      if (56*data->sino.no_frames!=Ang)
	{
	  fprintf(stderr,"Error reading file %s: Row dimension of sinogram doesn't equal 56*no_frames.\n\a\n",fname);
	  exit(-1);
	}
      data->sino.n_t=112;
      data->sino.n_theta=56*data->sino.no_frames;
      
      matClose(fin);


      if (cfg_info->NRM_FLAG)
	{
	  data->sino.nrm_FLAG=1; 
	  fin = matOpen(cfg_info->nrm_file, "r");
	  if (fin == NULL)
	    {
	      fprintf(stderr,"Cannot read MatLab normalization file %s\n",cfg_info->nrm_file);
	      exit(-1);
	    }

	  data->sino.nrm_img = read_mat_double_image(&Ang2,&T2,"counts",fin);
	  if (data->sino.nrm_img == NULL) 
	    {
	      fprintf(stderr,"Error reading normalization file %s: No counts array.\n\a\n",cfg_info->nrm_file);
	      exit(-1);
	    }
  
	  if (T2!=data->sino.n_t)/* || (Ang2*data->sino.no_frames!=Ang))*/
	    {
	      fprintf(stderr,"Normalization sinogram size doesn't match.\n\a\n");
	      exit(-1);
	    }
	  matClose(fin);
	}
    
    }
  else
    {
      /* Use IDL routines to read input and normalization */
      data->Y=cti_read_sino(cfg_info->sino_fname, cfg_info->no_sino, cfg_info->plane, &data->sino); 
    
      if (cfg_info->NRM_FLAG)
        {
          data->sino.nrm_img=read_nrm_plane(cfg_info->nrm_file, cfg_info->plane,  cfg_info->deadtime_FLAG);
          data->sino.nrm_FLAG=1;
        }
      else
         data->sino.nrm_FLAG=0;
     }


  /* RIGHT NOW, INITIAL COND, SUPPORT, TRANSMISSION-ATTN are always
     read as matlab files */

  /* Read initial X */  
  /*sprintf(fname,"%s_pl%d.mat",cmdline->initial,cmdline->plane);*/
  if (!strcmp(cfg_info->initial,"none"))
    {
      printf("No initial condition ('none'). Assuming constant 1e-2.\n");

      /*printf("INITIALIZING TO 6.05931 FOR BARPHANTOM !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\a\n");*/
      data->X=(double**)get_img(resol,resol, sizeof(double));
      for (i=0;i<resol;i++)
	for (j=0;j<resol;j++)
	  data->X[i][j]=0.01;
      /*data->X[i][j]=6.05931;*/
    }
  else
    {
      fin = matOpen(cfg_info->initial, "r");
      if (fin == NULL)
	{
	  fflush(stdout);fflush(stderr);
     	  fprintf(stderr,"\nError: Cannot read MatLab initial file %s.\nUse 'none' if you dont't want to use initial condition file.\n\a\n",cfg_info->initial);
	  exit(-1);
	}
    
      data->X = read_mat_double_image(&N,&M,"img",fin);
      if (data->X == NULL)
	{
	  fprintf(stderr,"Cannot read img matrix \n");
	  exit(-1);
	}
      matClose(fin);
      
      if ((N!=resol)||(M!=resol))
	{
	  fflush(stdout);fflush(stderr);
	  fprintf(stderr,"Size of initialization image (%d by %d ) doesn't match reconstruction size %d square.\n\a\n",N,M,resol);
	  
	  exit(-1);
	}
    }
  /* read support */
  if (cfg_info->SUP_FLAG)
   {
    fin = matOpen(cfg_info->mask_file, "r");
    if (fin == NULL)
      {
       fprintf(stderr,"Cannot read MatLab support file %s\n",cfg_info->mask_file);
       exit(-1);
      }
  
     temp_image = read_mat_double_image(&N,&M,"mask",fin);
     if (temp_image == NULL)
      {
        fprintf(stderr,"Cannot read mask matrix \n");
        exit(-1);
      }
    matClose(fin);
 
  
   if ((N!=resol)||(M!=resol))
      {
        fflush(stdout);fflush(stderr);
        fprintf(stderr,"Size of mask in sup-file (%d by %d ) doesn't match reconstruction size %d square.\n\a\n",N,M,resol);
        exit(-1);
      }

 
    /* convert double mask into integer mask */
    data->mask = (int **)get_img(M,N,sizeof(int));
    for(i=0;i<N*M;i++)
      data->mask[0][i] = (int)temp_image[0][i];
  
    free_img((void **)temp_image);
   }
  else
   {
    data->mask = (int **)get_img(resol,resol,sizeof(int)); 
    for(i=0;i<resol;i++)
      for (j=0;j<resol;j++)
        data->mask[i][j] = 1;
   }


  /* Read attenuation */
  if (cfg_info->attn_MODE == ATTN_FILE)
    {
      sfile_plane = 2*(int)(cfg_info->plane/9);
      if (9*(int)(cfg_info->plane/9) != cfg_info->plane)
	{
	  fprintf(stderr,"Error: Reading attenuation file for cross plane (%d) not implemented.\n\a\n",cfg_info->plane);
	  exit(-1);
	}
      data->sino.attn.sin_img=cti_read_attn_sin(cfg_info->attn_fname, sfile_plane, &data->sino.attn.img_rows, &data->sino.attn.img_cols);
      printf("N_ang=%d, M_offsets=%d\n",data->sino.attn.img_rows,data->sino.attn.img_cols);
     
    }
 

  
  
}
  

void write_output(struct cfg_info *cfg_info,struct all_data *data,double **sigma,struct geom_info *geom)
{
  MATFile *fout;
  int N,M,i,j;
  double **Xi;
  char matrix[20];
  double **tmp;
  
  tmp=(double**)multialloc(sizeof(double),2,1,1);

  N=geom->n_y;
  M=geom->n_x;
  printf("Writing output image %s\n",cfg_info->output);
  
  /* open file */
  /*sprintf(fname,"%s_pl%d.mat",cmdline->output,cmdline->plane);*/
  fout = matOpen(cfg_info->output, "w");
  if (fout == NULL)
    {
      fprintf(stderr,"Cannot write MatLab file %s\n",cfg_info->output);
      exit(-1);
    }
  
  /* write sigma */
  sprintf(matrix,"sigma");
  write_mat_double_image(sigma,1,1,matrix,fout);
  
  tmp[0][0]=cfg_info->fov;
  write_mat_double_image(tmp,1,1,"fov",fout);
  
  tmp[0][0]=data->sino.sep_d;
  write_mat_double_image(tmp,1,1,"sep_d",fout);
  tmp[0][0]=data->sino.dthck;
  write_mat_double_image(tmp,1,1,"dthck",fout);
  
  sprintf(matrix,"img");
  if (!cfg_info->INTERP_FLAG)
    {
      write_mat_double_image(data->X,N,M,matrix,fout);
    }
  else
    {
      printf("Interpolating output by 2...");
      Xi=(double**)multialloc(sizeof(double),2,2*N,2*M);
      
      for (i=0;i<N;i++)
	for (j=0;j<M;j++)
	  {
	    Xi[2*i][2*j]=Xi[2*i+1][2*j]=Xi[2*i][2*j+1]=Xi[2*i+1][2*j+1]=data->X[i][j];
	  }
      
      write_mat_double_image(Xi,2*N,2*M,matrix,fout);

      multifree((void*)Xi,2);
    }

  
  matClose(fout);
  multifree((void*)tmp,2);

}
  

 
 
void free_memory(struct all_data *data, struct column *sp_col,double **sigma,struct cfg_info *cfg_info)
{
  printf("Freeing memory ...\n");
  
  free_img((void **)data->X);
  free_img((void **)data->mask);
  free_img((void **)sigma);
  free_img((void **)data->Y);
  free((void *)sp_col->val);
  free((void *)sp_col->row);
  
 
  
  if (cfg_info->NRM_FLAG)
    free_img((void **)data->sino.nrm_img);
 
  if (cfg_info->attn_MODE==ATTN_FILE)
    multifree((void *)data->sino.attn.sin_img,2);
 
  if (cfg_info->MATLAB_FLAG==0)
    {
      multifree((void*)data->sino.frames_starttime,2);
      multifree((void*)data->sino.frames_duration,2);
     
      multifree((void*)data->sino.dtime.cor_singles,3);
      multifree((void*)data->sino.dtime.uncor_singles,3);
     
      free((void*)data->sino.dtime.scan_start_second);
      free((void*)data->sino.dtime.scan_start_minute);
      free((void*)data->sino.dtime.scan_start_hour);
      free((void*)data->sino.dtime.scan_start_day);
    }
}
 








void set_parameters(struct cmdline_params *cmdline,struct cfg_info *cfg_info,struct all_data *data,struct geom_info *geom)
{
  char *suf;
  int resol;
  double field_of_view;


  /* copy necessary cmdline params into cfg_info */
  strcpy(cfg_info->output,cmdline->output);
  strcpy(cfg_info->initial,cmdline->initial);
  cfg_info->plane = cmdline->plane;
  cfg_info->INTERP_FLAG = cmdline->INTERP_FLAG;
  
  /* copy optional cmdline params that override defaults in cfg_file */
  if (cmdline->no_iter_rec!=-1)
    cfg_info->no_iter_rec = cmdline->no_iter_rec;
  if (cmdline->resol!=-1)
    cfg_info->resol = cmdline->resol;
  if (cmdline->SIGMA_FLAG)
    cfg_info->sigma = cmdline->sigma;

  
  /* check if input is matlab or cti-scn file */
  suf=strrchr(cfg_info->sino_fname[0],'.');
  if (!strcmp(suf,".mat"))
    cfg_info->MATLAB_FLAG=1;
  else if (!strcmp(suf,".scn"))
    cfg_info->MATLAB_FLAG=0;
  else
    {
      fprintf(stderr,"Error: Specified sinogram filename %s doesn't have .mat or .scn suffix.\n\a\n",cfg_info->sino_fname[0]);
      exit(-1);
    }
  if (cfg_info->MATLAB_FLAG==1)
    {
      if (cfg_info->plane != 0)
	{
	  fprintf(stderr,"Error: plane=%d: Cannot specify plane in matlab mode.\n\a\n",cfg_info->plane);
	  exit(-1);
	}
    }


  /* copy params into sino: This should be changed in future, 
     instead of keeping copy in sino, should pass cfg_info
     to subroutines. */
  data->sino.DECAY_FLAG= cfg_info->decay_FLAG;
  data->sino.prj_max_t = cfg_info->prj_max_t;
  data->sino.analyt_sysresp = cfg_info->analyt_FLAG;
  data->sino.no_planes=1;
  data->sino.plane=cfg_info->plane;
  data->sino.DEADTIME_FLAG = cfg_info->deadtime_FLAG;
  data->sino.dtime.no_sino_files = cfg_info->no_sino;
  
  /* Set attenuation parameters */
  data->sino.attn.attn_analyt_r = cfg_info->attn_circ_rad;
  data->sino.attn.attn_cnst = cfg_info->attn_cnst;
  data->sino.attn.ATTN_MODE = cfg_info->attn_MODE;

  data->scatter = cfg_info->scatter;
  /*printf("Setting scatter to %g.\n",data->scatter);*/

  data->sino.frame_duration=0;
  data->sino.frame_start_time=0;


  resol=cfg_info->resol;
  field_of_view=cfg_info->fov; /* in cm  */
  geom->field_of_view=field_of_view;
  geom->n_y = resol;
  geom->n_x = resol;
  geom->delta =field_of_view/(double)geom->n_x;
  geom->dia = field_of_view; /* ?? */
  geom->x_0 = geom->y_0 = -0.5*((double)resol-1.0)*geom->delta;
  



}


void change_parameters_for_sgl_frame_recon(struct all_data *data, struct cmdline_params *cmdline)
{
  double inc_theta;
  int i;

  printf("Setting parameters for single frame reconstruction\n");
  if ((cmdline->single_frame<0)||(cmdline->single_frame>=data->sino.no_frames))
    {
      fprintf(stderr,"Error: Specified frame %d out of range [0, %d].\n\a\n", cmdline->single_frame,data->sino.no_frames-1);
      exit(-1);
    }
  if (cmdline->sglfr_th0cor_FLAG)
    {
      inc_theta=PI_OWN/data->sino.no_frames;
      data->sino.theta_0=-inc_theta*(double)cmdline->single_frame;
      printf("Calculating theta0 for single frame %d as theta_0=%f\n",cmdline->single_frame,data->sino.theta_0);
    } 
  
  
  data->sino.no_frames=1;
  data->sino.n_theta=56;
  for (i=0;i<6272;i++)
    data->Y[0][i]=data->Y[0][i+cmdline->single_frame*6272];
  printf("\n");
}
 

void print_configuration(struct cfg_info *cfg_info, struct cmdline_params *cmdline, struct all_data *data)
{
  if (cmdline->single_frame==-1)
     printf("Reconstructing all frames\n");
   else
     {
       printf("Reconstructing SINGLE frame %d, ",cmdline->single_frame);
       if (cmdline->sglfr_th0cor_FLAG)
	 printf("using theta_0 for that frame.\n");
       else
	 printf("not correcting for theta_0.\n");
     }

  printf("Scatter=%f\n",data->scatter);
  
  printf("Sino-config:\n");
  printf("n_theta = %d\nn_t=%d\ntheta_0=%f\nsep_d=%f\n",data->sino.n_theta,data->sino.n_t,data->sino.theta_0,data->sino.sep_d);
  printf("no_frames=%d\nnrm_FLAG=%d\nhalflife=%f\nDECAY_FLAG=%d\nplane=%d\nATTN_MODE=%d\n",data->sino.no_frames,data->sino.nrm_FLAG,data->sino.halflife,data->sino.DECAY_FLAG,data->sino.plane,data->sino.attn.ATTN_MODE);
  printf("attn_analyt_r=%f\nattn_cnst=%f\nprj_max_t=%f\nanalyt_sysresp=%d\nno_planes=%d\n",data->sino.attn.attn_analyt_r,data->sino.attn.attn_cnst,data->sino.prj_max_t,data->sino.analyt_sysresp,data->sino.no_planes);
  
 
  
  if (cfg_info->MATLAB_FLAG)
    printf("No Decay modeling in matlab mode.\n");
  else
    {
      if (data->sino.DECAY_FLAG)
	printf("Decay Modeling ON\n");
      else
	printf("Decay Modeling OFF\n");
    }

  printf("\n\n");

}

