#include <math.h> 
#include <stdio.h> 
#include <stdlib.h>
#include "ansi_lib_loc/io_mat_dbl.h"
#include "ansi_lib_loc/A_comp_iupui.h"
#include "ansi_lib_loc/allocate.h"
#include "ansi_lib_loc/randlib.h"

#define PI_OWN       3.14159265358979323846



int VERBOSE=1;
#define NOISY

void main(int argc, char **argv)
{
  /*FILE *fp;*/

  MATFile *fout,*fin;
  double **Y,**image,**nrm;
  int N,M,i,j;
  struct column sp_col;
  struct sino_info sino;
  struct geom_info geom;
  int proj_ind_1D,t;
  double **profiles=NULL;

  int nrm_FLAG=1;
  char *outfname;
  int resol,N_theta_nrm,N_t_nrm;
  double field_of_view;
  double **tmp;
  
  tmp=(double**)multialloc(sizeof(double),2,1,1);

 

  readseed();

  

  if ((argc<3) || (argc>4))
    {
      fprintf(stderr,"\nUsage: for_nrm emis_file [nrm_file] out_file\n\n");
      exit(-1);
    }


  if (argc==3)
    nrm_FLAG=0;
  outfname=argv[argc-1];
  

  /* Read input trn.mat file */
  fin = matOpen(argv[1], "r");
  if (fin == NULL) 
    {
      fprintf(stderr,"Cannot read MatLab file %s \n",argv[1]);
      exit(-1);
    }

  /* read counts matrix into Y */
  /* Routine  read_mat_double_image is in ansi_lib/io_mat_dbl.c */
  image = read_mat_double_image(&geom.n_x,&geom.n_y,"img",fin);
  if (image == NULL) 
    {
      image = read_mat_double_image(&geom.n_x,&geom.n_y,"X0",fin);
      if (image == NULL) 
	{
	  fprintf(stderr,"Cannot read img/X matrix \n");
	  exit(-1);
        }
    }
  /* Read n_theta, n_t, delta_theta, delta_t, theta_0, t_0,  dec_a=1, dec_t=1*/
  /* Routine in ansi_lib/Acomp.c */
  /*read_mat_sino_info(fin, &sino);*/
  matClose(fin);
  
  if (nrm_FLAG)
    {
      /* Read input nrm.mat file */
      fin = matOpen(argv[2], "r");
      if (fin == NULL) {
	fprintf(stderr,"Cannot read MatLab file %s \n",argv[2]);
	exit(-1);
      }
      
      nrm = read_mat_double_image(&N_theta_nrm,&N_t_nrm,"nrm",fin);
      if (nrm == NULL) 
	{
	  fprintf(stderr,"Cannot read nrm matrix \n");
	  exit(-1);
	}
      if ((N_t_nrm!=112)||(N_theta_nrm!=56))
	{
	  fprintf(stderr,"Cannot read nrm matrix has wrong size Nt=%d N_theta=%d, should be 112 by 56.\n\a \n",N_t_nrm,N_theta_nrm);
	  exit(-1);
	}
      
      matClose(fin);
      sino.nrm_FLAG=1;
      sino.nrm_img=nrm;
    }
  else
    {
      nrm=NULL;
      sino.nrm_FLAG=0;
      sino.nrm_img=NULL;
    }

  sino.DEADTIME_FLAG=0;
  sino.DECAY_FLAG=0;
  sino.attn.ATTN_MODE=0;
  /* As a first test, hardwire sino */
  /*sino.delta_t=3.1293;
    sino.delta_theta=0.0123;
    sino.n_t=192;
    sino.n_theta=256;
    sino.t_0=-298.8482;
    sino.theta_0=0.0;
  */
  
  resol=256;
  sino.frame_duration=sino.frame_start_time=0;
  sino.attn.attn_analyt_r=10.0;

  /* mouse_new */
  /*sino.prj_max_t=4.0; 
    field_of_view=8.0;*/

  /* mouse_large */
  sino.prj_max_t=9.0; 
  field_of_view=18.0;

  /* below: for mouse_for as we had 
  sino.prj_max_t=2.0; 
  field_of_view=4.0;  */


  geom.field_of_view=field_of_view;
  geom.n_y = resol;
  geom.n_x = resol;
  geom.delta =field_of_view/(double)geom.n_x;
  geom.dia = field_of_view; /* ?? */
  
  geom.x_0 = geom.y_0 = -0.5*((double)resol-1.0)*geom.delta;
  
 

  sino.sep_d=37.814999; /*30.252001;*/
  sino.no_frames=45;
  sino.theta_0=0.0;
  sino.n_t=112;
  sino.n_theta=56*sino.no_frames;

  init_A_matrix(&sino,&geom,"sysmat_p0");

  /* calc sino_arrays, also does cos correction of nrm-array */
  /*calc_sino_arrays(&sino);*/

  /* compute pixel profiles, data needed for computing A */
  /*pix_prof_comp(&sino,&geom,&profiles);*/ 
  
  /* Allocate memory for Acomp  - row */
  sp_col.val = (double *) get_spc(sino.n_t*sino.n_theta, sizeof(double));
  sp_col.row = (int *) get_spc(sino.n_t*sino.n_theta, sizeof(int));
  
  N = geom.n_y;
  M = geom.n_x;
  

  /********* Allocate space for Sinogram ************/
  Y = (double **)get_img(sino.n_t, sino.n_theta, sizeof(double));
  for(i=0; i<sino.n_t*sino.n_theta; i++)
    Y[0][i] = 0.0;
      

  /* backprojection */
  for(i=0; i<N; i++) 
    for(j=0; j<M; j++) 
      {
	A_comp(i,j,&sino, &geom, profiles,&sp_col,0);
	/*norm = 0.0;*/
	for(t=0; t<sp_col.n_row; t++) 
	  {
	    proj_ind_1D = sp_col.row[t];
	    Y[0][proj_ind_1D] += (image[i][j]*sp_col.val[t]);
	    
	    /*image[i][j] += sp_col.val[t]*Y[0][ii];*/
	    /*norm += sp_col.val[t];*/
	  }


	/*if (image[i][j] <= 0.0)
	  image[i][j] = 0.0;
	  else
	  image[i][j] = image[i][j]*PI/norm;*/
      }
  
  /*for(i=0; i<sino.n_t*sino.n_theta; i++)
    Y[0][i] = exp(-Y[0][i]);
  */
  
#ifdef NOISY
  /* add noise i.e. replace projection values by Poisson RV */
  /* with mean=proj.val */
  for (i=0;i<sino.n_theta*sino.n_t;i++)
    Y[0][i] = (double)poidev((float)Y[0][i]);
  
#endif


  fout = matOpen(outfname, "w");
  if (fout == NULL) 
    {
      fprintf(stderr,"Cannot write to MatLab file %s\n",outfname);
      exit(-1);
    }

  write_mat_double_image(Y,sino.n_theta,sino.n_t,"counts",fout);
  
  tmp[0][0]=sino.no_frames;
  write_mat_double_image(tmp,1,1,"no_frames",fout);
  tmp[0][0]=sino.sep_d;
  write_mat_double_image(tmp,1,1,"sep_d",fout);
  tmp[0][0]=sino.theta_0;
  write_mat_double_image(tmp,1,1,"theta_0",fout);
  tmp[0][0]=0.0;
  write_mat_double_image(tmp,1,1,"randoms",fout);



  matClose(fout);
  writeseed();

  free((void *)sp_col.val);
  free((void *)sp_col.row);
  /*free_img((void **)profiles);*/
  free_img((void **)image);
  free_img((void **)Y);
  if (nrm_FLAG)
    free_img((void **)nrm);
  multifree((void*)tmp,2);
}




