#include "fr_icm.h"
#include <stdio.h>
#include <math.h>
#include "ansi_lib_loc/allocate.h"
#include "ansi_lib_loc/assort.h"
/*#include "ansi_lib_loc/multigrid.h"*/
#include "ansi_lib_loc/solve2.h"
#include "ansi_lib_loc/randlib.h"


double ICMstep(struct ICMdata *ICMdata,struct all_data *data,double *S,int i,int j,struct geom_info *geom,struct column *sp_col,int *FLAG)
{	  
  double mm;
  int FLAG2; /* is =1 when all proj. Y through pixel == 0 */

  /* Compute D1 and D2 for quadratic approx. of liklihood ftn */
  *FLAG = compute_derivatives(ICMdata, data, sp_col,S,&FLAG2);

  if (FLAG2)
    {
      mm=0.0;
      printf("FLAG2\n\a\n");
      exit(-1);
    }
  else
    {
      if (ICMdata->D2<=0.0)
	{
	  fprintf(stderr,"Error in ICMstep: D2=%g <= 0.0\n\a\n",ICMdata->D2);
	  exit(-1);
	}
      
      /* compute ML extimate */
      ICMdata->ML = data->X[i][j] - ICMdata->D1/ICMdata->D2;
      /* get neighbors of (i,j) */
      get_neighborhood(ICMdata,data,geom,i,j);

      /* compute ICM-update */
      mm = updateX(ICMdata);
      /*printf("out of updateX\n");*/  
    }

  return(mm);
}


double updateX(struct ICMdata *ICMdata)
{
  double mm;
  double low,high;
  int status;

  /* calculate limits for half-interval search */
  minmax(&low, &high,ICMdata->ML,ICMdata->neigh.NX,ICMdata->neigh.DX);
  
  if (low<0.0)
    {
      low=0.0;
      if (high<low)
	high=low;
    }
 
 
  /*printf("in solve, low=%f high=%f, diffc(low)=%f diffc(high)=%f D1=%g D2=%g\n",low,high,diff_cost(low,(void*)(ICMdata)),diff_cost(high,(void*)(ICMdata)),ICMdata->D1, ICMdata->D2); */
  
  /* ICM */
  /* root derivative of approx. energy */
  /* this yields mm = mode of distribution */
  mm = solve(diff_cost, low, high, TOL, &status,(void*)(ICMdata));

  if (status)
    {
      if (cost(low,ICMdata) < cost(high,ICMdata))
	mm = low;
      else
	mm = high;
    }
  
  return(mm);
}




void get_neighborhood(struct ICMdata *ICMdata,struct all_data *data,struct geom_info *geom,int i,int j)
{
  int ai,aj,si,sj,N,M;
  double **X,**Y;
  

  X=data->X;
  Y=data->Y;
  N = geom->n_y;
  M = geom->n_x;


  ai = (i+1)%N;
  aj = (j+1)%M;
  si = i-1;
  if (si < 0)
    si = N-1;
  sj = j-1;
  if (sj < 0)
    sj = M-1;
	      
  ICMdata->neigh.NX[0] = X[i][aj];
  ICMdata->neigh.NX[1] = X[i][sj];
  ICMdata->neigh.NX[2] = X[ai][j];
  ICMdata->neigh.NX[3] = X[si][j];
  
  ICMdata->neigh.DX[0] = X[ai][aj];
  ICMdata->neigh.DX[1] = X[ai][sj];
  ICMdata->neigh.DX[2] = X[si][aj];
  ICMdata->neigh.DX[3] = X[si][sj];
  ICMdata->CX = X[i][j];

}




int compute_derivatives(struct ICMdata *ICMdata, struct all_data *data, struct column *sp_col,double *S, int *FLAG2)
{
  static double tmp,D1,D2;
  static int FLAG;
  static int t;/*,pro;*/
  static int *p_row;
  
  *FLAG2=1; /* return FLAG2=1: all proj. Y are zero */
  D1 = 0.0;
  D2 = 0.0;
  FLAG = 0;
  p_row=sp_col->row;
 
  for(t=0;t<sp_col->n_row;t++)
    {
      /*pro = sp_col->row[t];*/
      if ((S[*p_row]!= 0.0) && (data->Y[0][*p_row] != 0))
	{
	  D1 += sp_col->val[t] * (1.0 - data->Y[0][*p_row]/S[*p_row]);
          tmp=sp_col->val[t]/S[*p_row];
	  D2 += data->Y[0][*p_row]*tmp*tmp;
	  *FLAG2=0;
	}
      else if( data->Y[0][*p_row] == 0)
	D1 += sp_col->val[t];
      else
	{
	  FLAG = 1;
	  *FLAG2=0;
	}
      
      p_row++;
      
    }
  tmp=sp_col->multiplier;
  D1 *= tmp;
  D2 *= tmp*tmp;
  /*printf("FLAG=%d\n",FLAG);*/	
  ICMdata->D1=D1;
  ICMdata->D2=D2;
  
  return(FLAG);

	
}	

void updateS(double *S, struct column *sp_col, double diff)
{
  int t,pro;
  
  for(t=0;t<sp_col->n_row;t++)
    {
      pro = sp_col->row[t];
      S[pro] += sp_col->multiplier*sp_col->val[t]*diff;
    }
}



/* based on value of iter, uses different pixel order and returns locations i,j*/
/* for pixel number s*/
void imgcoord(int *out_i,int *out_j,int s,int iter,int N,int M)
{  
  int i,j,ii,jj,MM,NN,itemp;

  if((iter/4)%2 != 0)
    {
      NN = M;
      MM = N;
    }
  else
    {
      NN = N;
      MM = M;
    }
  
  ii=s/MM;
  jj=s-ii*MM;
  
  
  if((iter-1)%4 == 0)
    {
      i = NN-1-ii;
      j = MM-1-jj;
    }
  else 
    if((iter-2)%4 == 0)
      {
	i = NN-1-ii;
	j = jj;
      }
    else 
      if((iter-3)%4 == 0)
	{
	  i = ii;
	  j = MM-1-jj;
	}
      else
	{
	  i = ii;
	  j = jj;
	}
  
  if((iter/4)%2 != 0)
    {
      itemp = i;
      i = j;
      j = itemp;
    }
  
  *out_j=j;
  *out_i=i;

}
 





void minmax(double *low, double *high, double ML, double *NX, double *DX)
{
  int i;
  
  *low = ML;
  *high = ML;
  
  for(i=0;i<4;i++)
    {
      
      if( *low > NX[i])
	*low = NX[i];
      else if(*high < NX[i])
	*high = NX[i];
      
      if( *low > DX[i])
	*low = DX[i];
      else if(*high < DX[i])
	*high = DX[i];
      
    }
}




double diff_cost(double x,void *ICMdata)
{
    double prior=0.0, dprior=0.0;
    int i;
    struct ICMdata *par;
    
    par=(struct ICMdata*)ICMdata;
    
    
    
    for(i=0;i<4;i++)
      {
	prior += pow(fabs(x - par->neigh.NX[i]),par->P)*SG(x,par->neigh.NX[i]);
	dprior += pow(fabs(x - par->neigh.DX[i]),par->P)*SG(x,par->neigh.DX[i]);
      }
    prior  = prior*WN;
    dprior  = dprior*WD;
    
    return( par->D1 + par->D2*(x - par->CX) + (prior+dprior)/par->SX );
}



double cost(double x,struct ICMdata *par)
{
  double temp, prior=0.0, dprior=0.0;
  int i;
  
  temp = x-par->CX;
  
  for(i=0;i<4;i++)
    {
      prior += pow(fabs(x - par->neigh.NX[i]),par->P+1);
      dprior += pow(fabs(x - par->neigh.DX[i]),par->P+1);
    }
  prior  = prior*WN;
  dprior  = dprior*WD;
  
  return( par->D1*temp + par->D2*temp*temp/2.0 + (prior + dprior )/((par->P+1)*par->SX) );
}



void  f_pro(double *S,struct all_data *data,struct geom_info *geom,double **profiles,struct column *sp_col)
{
  int i,j,t;
  int M,N;


  N = geom->n_y;
  M = geom->n_x;
 
  /* Adding scatter */
  for(i=0;i<data->sino.n_t*data->sino.n_theta;i++)
    S[i] = data->scatter;
  
  for(i=0;i<N;i++)
    for(j=0;j<M;j++)
      {
	A_comp(i,j,&data->sino,geom,profiles,sp_col,0);
        for(t=0;t<sp_col->n_row; t++)
	  S[sp_col->row[t]] += sp_col->multiplier*sp_col->val[t]*data->X[i][j];
      }
}




