#include "fr_recon.h"

/*#define WRITE_INTERMED_RESULT*/




double relax_recon(struct all_data *data,double *S,struct geom_info *geom, double **profiles, struct cfg_info *cfg_info,struct column *sp_col,double P,double SX)
{
  
  int i,j,s,iter,N,M;
  int FLAG;
  double diff,new;
  double **X,**Y;
  double l_like;
  struct ICMdata ICMdata;
  

#ifdef WRITE_INTERMED_RESULT
  char matrix[20];
  MATFile *fout;
  double **it;
#endif

  X=data->X;
  Y=data->Y;
  N = geom->n_y;
  M = geom->n_x;
   
  /* ICM data */
  ICMdata.SX=SX;
  ICMdata.P=P;

  for(iter=0;iter<cfg_info->no_iter_rec;iter++)
    {
#ifdef WRITE_INTERMED_RESULT
      if (6*(iter/6)==iter)
	{	
          fout = matOpen("intermed.mat", "w");
	  if (fout == NULL)
	    {
	      fprintf(stderr,"Cannot write MatLab file intermed.mat\n");
	      exit(-1);
	    }
	  sprintf(matrix,"img");
	  write_mat_double_image(data->X,N,M,matrix,fout);

	  it=(double**)multialloc(sizeof(double),2,1,1);
	  it[0][0]=(double)iter;
	  sprintf(matrix,"iter");
	  write_mat_double_image(it,1,1,matrix,fout);
	  matClose(fout);
	  multifree((void**)it,2);
	 }
#endif 


      for (s=0;s<N*M;s++) /* loop over pixel locations */
	{
	  /* determine i,j image location, depending on iter go in*/
	  /* row or column order and also start from different corners */
	  imgcoord(&i,&j,s,iter,N,M);

	  /* compute column of A matrix for loc. i,j */
	  A_comp(i,j,&data->sino,geom,profiles,sp_col,0);

	  
	  /* if more than 0 projections intersect location i,j */
	  if(sp_col->n_row)
	    {
	      new=ICMstep(&ICMdata,data,S,i,j,geom,sp_col,&FLAG);

	      /* In reconstruction mode, use ICM update , clip to >= 0.0 */
	      if( new < 0.0)
		new = 0.0;

	      X[i][j]=new;
	      
	      /* if  FLAG not zero -> there is nonzero proj. intersecting w/ i,j, but X[i][j] got clipped to zero */
	      if ( FLAG && X[i][j] == 0.0)
		X[i][j] = 1.0/sp_col->n_row;
	      
	      /* If pixel-value was changed, update S=Ax */
	      diff = X[i][j] - ICMdata.CX;
	      if(diff)
		updateS(S,sp_col,diff);

	     
	    } /* end if sp_col */
	} /* end for s */
      
     
      l_like=calc_llikelihood(data,geom, S, P, ICMdata.SX);
      printf("%d. Normalized log-likelihood %f\n",iter+1,l_like/(N*M));
      fflush(stdout);fflush(stderr);
      
    } /* end for iter */
  
  return(ICMdata.SX);
}	





double calc_llikelihood(struct all_data *data,struct geom_info *geom, double *S, double P, double SX)
{
  double l_like = 0.0;
  int i;
  int M,N;
  
  N = geom->n_y;
  M = geom->n_x;    
    
  for(i=0;i<data->sino.n_t*data->sino.n_theta;i++)
    if ( S[i] > 0.0 )
      l_like += -S[i] + data->Y[0][i]*log( S[i] );

   

  
  l_like -= energy8_toroid_dbl(data->X,N,M,P+1)/((P+1)*SX);
  
  return(l_like);
}




