#include "iu_utils.h"

void y_mask(struct all_data *data)
{
  double **Y;
  int fr,i,j,no_frames;
  int det1[112][56],det2[112][56];

  no_frames=data->sino.no_frames;
  Y=data->Y;

  /* first, find minimum of sinogram counts in diamond only */
  for (i=0;i<56;i++)
    for (j=0;j<112;j++)
      {
	det1[j][i] = ( (int)(84.0+(double)i+((double)j)/2.0) )%112; /* dbot*/
	det2[j][i] = ( (int)(84.0+(double)i-((double)j)/2.0) )%112; /* dtop*/

	if ((det1[j][i]>55)||(det2[j][i]<56))
	  for (fr=0;fr<no_frames;fr++)
	    Y[fr*56+i][j]=0.0;
	  
      }
}


void y_positivity(struct all_data *data)
{
  double min;
  double **Y;
  int fr,i,j,no_frames;
  int det1[112][56],det2[112][56];

  min=0.0;
  
  no_frames=data->sino.no_frames;
  Y=data->Y;

  /* first, find minimum of sinogram counts in diamond only */
  for (i=0;i<56;i++)
    for (j=0;j<112;j++)
      {
	det1[j][i] = ( (int)(84.0+(double)i+((double)j)/2.0) )%112; /* dbot*/
	det2[j][i] = ( (int)(84.0+(double)i-((double)j)/2.0) )%112; /* dtop*/

	if ((det1[j][i]<=55)&&(det2[j][i]>=56))
	  {
	    for (fr=0;fr<no_frames;fr++)
	      if (Y[fr*56+i][j]<min)
		min=Y[fr*56+i][j];
	  }
      }
  
  printf("Minimum sinogram entry in diamond is %f.\n",min);
min/=3.0; 

  /* if min<0, add -min to sinogram */
  if (min<0.0)
    {
      printf("*********************** Adding %f to the sinogram.\n\n",-min);
      for (fr=0;fr<no_frames;fr++)
	for (i=0;i<56;i++)
	  for (j=0;j<112;j++)
	    Y[fr*56+i][j] -= min;
    }

  /* NOTE: Entries outside the diamond can and probably will
     still be negative. I assume those are NOT being used. */
  data->scatter -=min;
  printf("New scatter value =%f\n",data->scatter);


}


void clip_y(struct all_data *data)
{
  double **Y;
  int fr,i,j,no_frames;
 
  no_frames=data->sino.no_frames;
  Y=data->Y;

  for (fr=0;fr<no_frames;fr++)
    for (i=0;i<56;i++)
      for (j=0;j<112;j++)
	if (Y[fr*56+i][j]<1.0e-6)
	  Y[fr*56+i][j]=1.0e-6;
}



void truncate_init(struct all_data *data, struct geom_info *geom)
{
  int i,j,M,N;
  double r_sq,x,y;

  N=geom->n_y;
  M=geom->n_x;
    
  /* Now eliminate activity outside prj_max_t in initial condition */
  r_sq=data->sino.prj_max_t*data->sino.prj_max_t;
  for (i=0;i<N;i++)
    for (j=0;j<M;j++)
      {
	y = geom->y_0 + (double)i * geom->delta;
	x = geom->x_0 + (double)j * geom->delta;
	if (x*x+y*y > r_sq)
	  data->X[i][j]=1e-6;
      }
}

void print_decay_info(struct all_data *data)
{
  int i;

  printf("Modeling exponential decay:\n");
  printf("  Frame-Starttimes in sec = [");
  for (i=0;i<data->sino.no_frames;i++)
    printf("%d  ",data->sino.frames_starttime[0][i]);
  printf("]\n");
  
  printf("  Frame-durations in sec = [");
  for (i=0;i<data->sino.no_frames;i++)
    printf("%d  ",data->sino.frames_duration[0][i]);
  printf("]\n");
  printf("  Halflife in sec = %f\n",data->sino.halflife);
}


void write_Ycor(struct all_data *data)
{
  int pr,ind,pind,i;
  FILE *fp,*fp2;
  double nrm,dec,out;
  
  fp=fopen("Ycor","w");
  fp2=fopen("Y","w");
  if (fp==NULL)
    {
      fprintf(stderr,"Can't write Ycor.\n\a\n");
      exit(-1);
    }
  
  for(pr=0; pr < data->sino.n_theta; pr++)
    {
      for(i = 0; i < data->sino.n_t; i++) 
	{   
	  pind = pr*data->sino.n_t;
	  ind = pind+i; /* linear index in 1D array*/
	  
	  out=data->Y[pr][i];
	  
	  if (data->sino.nrm_FLAG)
	    {
	      nrm=(double)data->sino.nrm_arr[ind];
	      if (nrm<1e-5)
		nrm=60.0;
	      out /= nrm;
	    }
	  
	  if (data->sino.DECAY_FLAG)
	    {
	      dec=(double)data->sino.decay_arr[ind];
	      out /= dec;
	    }
	  
	  fprintf(fp,"%1.20g ",out);
	  fprintf(fp2,"%1.20g ",data->Y[pr][i]);
	}
      fprintf(fp,"\n");
      fprintf(fp2,"\n");
    }
  
  fclose(fp);
  fclose(fp2);
}
  
