#include <stdio.h>
#include <math.h>
#include "fr_recon.h"
#include "fr_paramest.h"
#include "ansi_lib_loc/io_mat_dbl.h"
#include "ansi_lib_loc/cti_io.h"
#include "ansi_lib_loc/allocate.h"
#include "ansi_lib_loc/A_comp_iupui.h"
/*#include "fr_icm.h"
 */
#ifdef __linux__
#include <getopt.h>
#endif
extern int optind;
extern char *optarg;
   


void main(int argc, char **argv)
{
    
  MATFile *fin;  
  struct sino_info sino;
  struct geom_info geom;
  double **X;
  int mfile;
  mxArray *pm;

  
  int N,M,N1=0,M1=0;
  
  if (argc<3)
    {
      fprintf(stderr,"Usage: mat2img <mat-file1> ... <mat-file N> <out.img>\n\n");
      exit(-1);
    }

  sino.no_planes=argc-2;
  
  sino.frame_duration=1.0;
  sino.frame_start_time=0.0;
  
  /*printf("Manually setting parameters:\n");
  printf("frame_duration= %d\n",(int)sino.frame_duration);
  printf("frame_start_time= %d\n",(int)sino.frame_start_time);
  printf("\n\n");
  */
  
  for (mfile=0;mfile<sino.no_planes;mfile++)
    {
      printf("Converting file %s into plane %d.\n",argv[mfile+1],mfile);
      /* read input file*/
      fin = matOpen(argv[mfile+1], "r");
      if (fin == NULL)
	{
	  fprintf(stderr,"Cannot read MatLab input file %s\n",argv[mfile+1]);
	  exit(-1);
	}

      pm = matGetArray(fin,"sep_d");
      if(pm != NULL) 
	sino.sep_d = mxGetScalar(pm);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable sep_d.\n\a\n",argv[mfile+1]);
	  exit(-1);
	}
      mxDestroyArray(pm);
       

      pm = matGetArray(fin,"fov");
      if(pm != NULL) 
	geom.field_of_view = mxGetScalar(pm);
      else
	{
	  fprintf(stderr,"Error reading file %s: No variable fov.\n\a\n",argv[mfile+1]);
	  exit(-1);
	}
      mxDestroyArray(pm);
      
      printf("fov=%f  sep_d=%f\n",geom.field_of_view,sino.sep_d);
      

  
      X = read_mat_double_image(&N,&M,"img",fin);
      if (X == NULL)
	{
	  fprintf(stderr,"Cannot read img matrix \n");
	  exit(-1);
	}
      matClose(fin);
      geom.n_y=N;geom.n_x=M;
      printf("Size is %d by %d pixels\n",N,M);
      if (N!=M)
	{
	  fprintf(stderr,"Warning: This programs was never tested for non-square arrays, row and col sizes might be interchanged.\n");
	}


      if (mfile==0)
	{
	  N1=N;
	  M1=M;
	}
      else
	{
	  if ((N1!=N)||(M1!=M))
	    {
	      fprintf(stderr,"WARNING: Matrix size is different from size of first matrix.\n\a\n");
	      exit(-1);
	    }
	}

      
      
      if (mfile==0)
	write_cti_image_file(argv[argc-1], X, N, M, &sino, &geom);
      else
	add_cti_image_mat(argv[argc-1], X, N, M, &sino, &geom,mfile);
      
      free_img((void **)X);

    }
}
