#include "read_cfg_file.h"

static void checkstat(int stat,int num);


static void checkstat(int stat,int num)
{
  if (stat!=num)
    {
      fprintf(stderr,"Read error in configuration file: stat=%d, should be %d.\n\a\n",stat,num);
      exit(-1);
    }
}


void read_cfg_file(char *fname,struct cfg_info *cfg_info)
{
  FILE *fp;
  int got_no_sino=0,got_sino_fname=0, got_nrm=0,got_mask=0,got_fov=0;
  int got_def_res=0,got_def_iter=0,got_param_iter=0,got_sig=0,got_p=0;
  int got_analyt=0,got_fnsysres=0,got_addposi=0,got_attn_flag=0;
  int got_attnrad=0,got_decay_cor=0,got_max_disp=0,got_scatter=0;
  int got_verb=0,got_deadtime=0,got_attncnst=0,got_attnfname=0;
  char st1[256],st2[256];
  int stat,i;

  fp=fopen(fname,"r");
  if (fp==NULL)
    {
      fprintf(stderr,"Can't open cfg_info %s.\n\a\n",fname);
      exit(-1);
    }

  stat=fscanf(fp,"%s",st1);checkstat(stat,1);
  if (strcmp(st1,"ConfigFileEmisnew"))
    {
      fprintf(stderr,"Config file %s doesn't start with 'ConfigFileEmisnew'.\n\a\n",fname);
      exit(-1);
    }
  
  for (;;)
    {
      stat=fscanf(fp,"%s",st1);checkstat(stat,1);
      if (!strcmp(st1,"EndConfigFile"))
	break;
      else if (!strcmp(st1,"number_sinogram_files:"))
	{
	  stat=fscanf(fp,"%d",&cfg_info->no_sino);checkstat(stat,1);
	  got_no_sino=1;
	}      
      else if (!strcmp(st1,"sinogram_files:"))
	{
	  if (got_no_sino==0)
	    {
	      fprintf(stderr,"Error reading cfg-file %s: 'number_sinogram_files' must be before 'sinogram_files'\n\a\n",fname);
	      exit(-1);
	    }
	  cfg_info->sino_fname = (char**)multialloc(sizeof(char),2,cfg_info->no_sino,256);
	  for (i=0;i<cfg_info->no_sino;i++)
	    {
	      stat=fscanf(fp,"%s",cfg_info->sino_fname[i]);checkstat(stat,1);
	    }
	  got_sino_fname=1;
	}  
      else if (!strcmp(st1,"normalization_file:"))
	{
	  stat=fscanf(fp,"%s",cfg_info->nrm_file);checkstat(stat,1);
	  got_nrm=1;
	  if (!strcmp(cfg_info->nrm_file,"off"))
	    cfg_info->NRM_FLAG=0;
	  else
	    cfg_info->NRM_FLAG=1;
	}      
      else if (!strcmp(st1,"support_mask_file_for_sigma_est:"))
	{
	  stat=fscanf(fp,"%s",cfg_info->mask_file);checkstat(stat,1);
	  got_mask=1;
	  if (!strcmp(cfg_info->mask_file,"off"))
	    cfg_info->SUP_FLAG=0;
	  else
	    cfg_info->SUP_FLAG=1;
	}      
      else if (!strcmp(st1,"field_of_view_reconstruction_cm:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->fov);checkstat(stat,1);
	  got_fov=1;
	}
      else if (!strcmp(st1,"default_resolution:"))
	{
	  stat=fscanf(fp,"%d",&cfg_info->resol);checkstat(stat,1);
	  got_def_res=1;
	}
      else if (!strcmp(st1,"default_no_iterations:"))
	{
	  stat=fscanf(fp,"%d",&cfg_info->no_iter_rec);checkstat(stat,1);
	  got_def_iter=1;
	}
      else if (!strcmp(st1,"default_sigma:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->sigma);checkstat(stat,1);
	  got_sig=1;
	}
      else if (!strcmp(st1,"MRF_prior_parameter_p:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->p);checkstat(stat,1);
	  got_p=1;
	}
      else if (!strcmp(st1,"parameter_estimation_iterations:"))
	{
	  stat=fscanf(fp,"%d",&cfg_info->no_iter_param);checkstat(stat,1);
	  got_param_iter=1;
	}
      else if (!strcmp(st1,"use_analytical_system_response:"))
	{
	  stat=fscanf(fp,"%s",st2);checkstat(stat,1);
	  got_analyt=1;
	  if (!strcmp(st2,"on"))
	    cfg_info->analyt_FLAG=1;
	  else if  (!strcmp(st2,"off"))
	    cfg_info->analyt_FLAG=0;
	  else
	    {
	      fprintf(stderr,"Error reading cfg-file %s: illegal value %s for use_analytical_system_response. Must be 'on' or 'off'.\n\a\n",fname,st2);
	      exit(-1);
	    }
	}
      else if (!strcmp(st1,"system_response_file:"))
	{
	  stat=fscanf(fp,"%s",cfg_info->sysres_fname);checkstat(stat,1);
	  got_fnsysres=1;
	}
      else if (!strcmp(st1,"add_sino_positivity_cnst:"))
	{
	  stat=fscanf(fp,"%s",st2);checkstat(stat,1);
	  got_addposi=1;
	  if (!strcmp(st2,"on"))
	    cfg_info->addposi_FLAG=1;
	  else if  (!strcmp(st2,"off"))
	    cfg_info->addposi_FLAG=0;
	  else
	    {
	      fprintf(stderr,"Error reading cfg-file %s: illegal value %s for add_sino_positivity_cnst. Must be 'on' or 'off'.\n\a\n",fname,st2);
	      exit(-1);
	    }
	}
      else if (!strcmp(st1,"attenuation_correction:"))
	{
	  stat=fscanf(fp,"%s",st2);checkstat(stat,1);
	  got_attn_flag=1;
	  if (!strcmp(st2,"file"))
	    cfg_info->attn_MODE=ATTN_FILE;
	  else if (!strcmp(st2,"circle"))
	    cfg_info->attn_MODE=ATTN_CIRCLE;
	  else if  (!strcmp(st2,"off"))
	    cfg_info->attn_MODE=0;
	  else
	    {
	      fprintf(stderr,"Error reading cfg-file %s: illegal value %s for attenuation_correction. Must be 'on' or 'off'.\n\a\n",fname,st2);
	      exit(-1);
	    }
	}
      else if (!strcmp(st1,"attenuation_circle_radius_cm:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->attn_circ_rad);checkstat(stat,1);
	  got_attnrad=1;
	}
      else if (!strcmp(st1,"attenuation_filename:"))
	{
	  stat=fscanf(fp,"%s",cfg_info->attn_fname);checkstat(stat,1);
	  got_attnfname=1;
	}
      else if (!strcmp(st1,"attenuation_constant_invcm:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->attn_cnst);checkstat(stat,1);
	  got_attncnst=1;
	}
      else if (!strcmp(st1,"decay_correction:"))
	{
	  stat=fscanf(fp,"%s",st2);checkstat(stat,1);
	  got_decay_cor=1;
	  if (!strcmp(st2,"on"))
	    cfg_info->decay_FLAG=1;
	  else if  (!strcmp(st2,"off"))
	    cfg_info->decay_FLAG=0;
	  else
	    {
	      fprintf(stderr,"Error reading cfg-file %s: illegal value %s for decay_correction. Must be 'on' or 'off'.\n\a\n",fname,st2);
	      exit(-1);
	    }
	}
      else if (!strcmp(st1,"max_displacement_prj_in_cm:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->prj_max_t);checkstat(stat,1);
	  got_max_disp=1;
	}
      else if (!strcmp(st1,"scatter_parameter:"))
	{
	  stat=fscanf(fp,"%lf",&cfg_info->scatter);checkstat(stat,1);
	  got_scatter=1;
	}
      else if (!strcmp(st1,"dead_time_correction:"))
	{
	  stat=fscanf(fp,"%s",st2);checkstat(stat,1);
	  got_deadtime=1;
	  if (!strcmp(st2,"on"))
	    cfg_info->deadtime_FLAG=1;
	  else if  (!strcmp(st2,"off"))
	    cfg_info->deadtime_FLAG=0;
	  else
	    {
	      fprintf(stderr,"Error reading cfg-file %s: illegal value %s for dead_time_correction. Must be 'on' or 'off'.\n\a\n",fname,st2);
	      exit(-1);
	    }
	}
      else if (!strcmp(st1,"verbose:"))
	{
	  stat=fscanf(fp,"%s",st2);checkstat(stat,1);
	  got_verb=1;
	  if (!strcmp(st2,"on"))
	    cfg_info->verbose=1;
	  else if  (!strcmp(st2,"off"))
	    cfg_info->verbose=0;
	  else
	    {
	      fprintf(stderr,"Error reading cfg-file %s: illegal value %s for verbose. Must be 'on' or 'off'.\n\a\n",fname,st2);
	      exit(-1);
	    }
	}
      else 
	{
	  fprintf(stderr,"Error reading cfg-file %s: Unknown entry %s.\n\a\n",fname,st1);
	  exit(-1);
	}
    }


  /* Now check if we got everything */
  if (got_no_sino!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: number_sinogram_files missing.\n\a\n",fname);
      exit(-1);
    }
  if (got_sino_fname!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: sinogram_files  missing.\n\a\n",fname);
      exit(-1);
    }
  if (got_nrm!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: normalization_file missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_mask!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: support_mask_file_for_sigma_est missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_fov!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: field_of_view_reconstruction_cm missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_def_res!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: default_resolution missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_def_iter!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: default_no_iterations missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_param_iter!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: parameter_estimation_iterations missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_sig!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: default_sigma missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_p!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: MRF_prior_parameter_p missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_analyt!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: use_analytical_system_response missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_fnsysres!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: system_response_file missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_addposi!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: add_sino_positivity_cnst missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_attn_flag!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: attenuation_correction missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_attnfname!=1)
     {
       fprintf(stderr,"Error reading cfg-file %s: attenuation_filename missing.\n\a\n",fname);
       exit(-1);
     }  
   if (got_attnrad!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: attenuation_circle_radius_cm missing.\n\a\n",fname);
      exit(-1);
    }
    if (got_attncnst!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: attenuation_constant_invcm missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_decay_cor!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: decay_correction missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_max_disp!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: max_displacement_prj_in_cm missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_scatter!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: scatter_parameter missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_deadtime!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: dead_time_correction missing.\n\a\n",fname);
      exit(-1);
    }
   if (got_verb!=1)
    {
      fprintf(stderr,"Error reading cfg-file %s: verbose missing.\n\a\n",fname);
      exit(-1);
    }
   


}


void print_cfg_info(struct cfg_info *cfg_info)
{
  int i;
  
  printf("CFG-INFO:\n");
  printf("Number of sinogram files: %d\nFilenames:\n",cfg_info->no_sino);
  for (i=0;i<cfg_info->no_sino;i++)
    printf("%s\n",cfg_info->sino_fname[i]);
  
  if (cfg_info->NRM_FLAG)
    printf("Using normalization correction, nrm_file=%s\n",cfg_info->nrm_file);
  else
    printf("NOT using normalization correction\n");
  if (cfg_info->SUP_FLAG)
    printf("Using mask, mask_file=%s\n",cfg_info->mask_file);
  else
    printf("NOT using mask for sigma-estimation.\n");
  printf("field of view = %f\nresolution = %d\nno_iter_rec = %d\n",cfg_info->fov,cfg_info->resol,cfg_info->no_iter_rec);
  printf("no_iter_param = %d\nsigma=%f\np=%f\n",cfg_info->no_iter_param,cfg_info->sigma, cfg_info->p);
  
  if (cfg_info->analyt_FLAG)
    printf("Using analytical system response even if sysresp. file contains measured data.\n");
  else
    printf("Using system response specified in response file.\n");
  
  printf("System response file: %s\n",cfg_info->sysres_fname);
  
  if (cfg_info->addposi_FLAG)
    printf("Adding positivity constant to sinogram\n");
  else
    printf("NOT Adding positivity constant to sinogram\n");
  
  
  if (cfg_info->decay_FLAG)
    printf("Modeling DECAY\n");
  else
    printf("NOT modeling DECAY\n");
  
  printf("Use projections w/ displacement t < %f\n",cfg_info->prj_max_t);
  
  printf("SCATTER = %f\n",cfg_info->scatter);
  
  if (cfg_info->deadtime_FLAG)
    printf("Deadtime correction ON\n");
  else
    printf("Deadtime correction OFF\n");
  
  printf("Matlab mode %d\n",cfg_info->MATLAB_FLAG);
  printf("Reconstructing PLANE %d\n",cfg_info->plane);
  
  printf("Initial condition %s\n",cfg_info->initial);
  
  if (cfg_info->attn_MODE==ATTN_FILE)
    printf("Using attenuation correction from file %s\n",cfg_info->attn_fname);
  else if (cfg_info->attn_MODE==ATTN_CIRCLE)
    {
      printf("Use attenuation correction CIRCLE model\n");
      printf("attn-circle-radius = %f cm, attn_cnst=%f\n",cfg_info->attn_circ_rad,cfg_info->attn_cnst);
    }
  else if (cfg_info->attn_MODE==0)
    printf("NOT using attenuation correction from file\n");
  else
    {
      fprintf(stderr,"Error: Invalid attn_MODE=%d\n\a\n",cfg_info->attn_MODE);
      exit(-1);
    }

  if  (cfg_info->INTERP_FLAG)
    printf("Interpolating output by a factor of 2\n");
  else
    printf("Not interpolating output\n");
   
  printf("output file = %s\n",cfg_info->output);
  printf("\n\n");
  
}


void check_parameters(struct cfg_info *cfg_info)
{
  
  if (cfg_info->MATLAB_FLAG)
    {
      if (cfg_info->plane!=0)
	{
	  fprintf(stderr,"Error: Can't specify plane in Matlab mode.\n\a\n");
	  exit(-1);
	}

      if (cfg_info->no_sino != 1)
	{
	  fprintf(stderr,"Error: Can't use multiple input files in matlab mode\n\a\n");
	  exit(-1);
	} 
      
      if (cfg_info->decay_FLAG != 0)
	{
	  fprintf(stderr,"Error: Don't support decay correction in matlab mode.\n\a\n");
	  exit(-1);
	} 

    }
      
 
  if (cfg_info->prj_max_t > cfg_info->fov/2.0)
    {
      fprintf(stderr,"Error: specified max displacement prj_max_t of %f cm is larger than fov/2.0 = %f cm.\n",cfg_info->prj_max_t,cfg_info->fov/2.0);
      exit(-1);
    }


  if ((cfg_info->attn_MODE == ATTN_CIRCLE) && (strcmp(cfg_info->attn_fname,"none")))
    {
      fprintf(stderr,"Error: When attenuation_correction=circle , must set attn_filname=none, set to %s\n\a\n", cfg_info->attn_fname);
      exit(-1);
    }

  
} 

