#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


#ifdef __linux__
#include <getopt.h>
#endif
extern int optind;
extern char *optarg;



struct cmdline_loc{
  char outfile[256];
  char cfg_file[256];
  char init_file[256];
  int resol;
  int no_iter_rec;
  int INTERP_FLAG;
  int SIGMA_FLAG;
  double sigma;
  int single_frame;
  int sglfr_th0cor_FLAG;
  int first_plane,last_plane,step_plane;

};

void cmdline_error();
void parse_cmdline(struct cmdline_loc *cmdline,int argc,char **argv);



void main(int argc, char **argv)
{
  int plane;
  char cmd[512],base_cmd[256];
  struct cmdline_loc cmdline;

  parse_cmdline(&cmdline,argc,argv);
  printf("first plane=%d, last plane=%d, step=%d.\n\n",cmdline.first_plane,cmdline.last_plane,cmdline.step_plane);

  sprintf(base_cmd,"./emisnew");
  if (cmdline.resol!=-1)
    sprintf(base_cmd,"%s -r %d",base_cmd,cmdline.resol);
  if (cmdline.no_iter_rec!=-1)
    sprintf(base_cmd,"%s -i %d",base_cmd,cmdline.no_iter_rec);
  if (cmdline.INTERP_FLAG)
    sprintf(base_cmd,"%s -I",base_cmd);
  if (cmdline.SIGMA_FLAG)
    sprintf(base_cmd,"%s -s %f",base_cmd,cmdline.sigma);
  if (cmdline.single_frame!=-1)
    sprintf(base_cmd,"%s -f %d",base_cmd,cmdline.single_frame);
  if (cmdline.sglfr_th0cor_FLAG)
    sprintf(base_cmd,"%s -t",base_cmd);
  

  for (plane=cmdline.first_plane;plane<=cmdline.last_plane;plane+=cmdline.step_plane)
    {
      if (!strcmp(cmdline.init_file,"none"))
	sprintf(cmd,"%s -p %d %s none %s_p%d.mat",base_cmd,plane,cmdline.cfg_file,cmdline.outfile,plane);
      else
	sprintf(cmd,"%s -p %d %s %s_p%d.mat %s_p%d.mat",base_cmd,plane,cmdline.cfg_file,cmdline.init_file,plane,cmdline.outfile,plane);
      
      printf("Executing: %s\n\n",cmd);
      fflush(stdout);fflush(stderr);
      system(cmd);
      fflush(stdout);fflush(stderr);
    }


  /* Now that we have reconstructed all planes, generate a single 
     cti-img output file */
  fflush(stderr);fflush(stdout);
  printf("Merging single plane recons into single cti-img file...\n");fflush(stdout);
  sprintf(cmd,"./mat2img");
  for (plane=cmdline.first_plane;plane<=cmdline.last_plane;plane+=cmdline.step_plane)
    sprintf(cmd,"%s %s_p%d.mat",cmd,cmdline.outfile,plane);
  sprintf(cmd,"%s %s.img",cmd,cmdline.outfile);
  printf("Executing: %s\n",cmd);fflush(stdout);fflush(stderr);
  system(cmd);

}


void cmdline_error()
{
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"recon_all_planes [-r recon_resolution] [-i no_iter] [-I (interpolate result)] [-s sigma] [-p first_plane (default 0)] [-l last_plane (default 63)] [-n plane_step (default 9) [-f singleframe_no [-t]] cfg_file init_file out_file\n\n");
  exit(-1);  
}


void parse_cmdline(struct cmdline_loc *cmdline,int argc,char **argv)
{
  int i,ct;
  int stat_res=0;
  
  cmdline->first_plane=0;
  cmdline->last_plane=63;
  cmdline->step_plane=9;
 

  cmdline->single_frame=-1;
  cmdline->sglfr_th0cor_FLAG=0;
  cmdline->INTERP_FLAG=0;
  cmdline->SIGMA_FLAG=0;
  cmdline->no_iter_rec=-1;
  cmdline->resol=-1;
  
  for (;;)
    { 
      i= getopt(argc, argv, "p:r:i:Is:f:t:l:n:");
      if (i==EOF)
        break;
      else
        {
          if (i == '?')
            cmdline_error();
	  if (i=='p')
            {
	      cmdline->first_plane = atoi(optarg);
            }
	  if (i=='l')
            {
	      cmdline->last_plane = atoi(optarg);
            }
	  if (i=='n')
            {
	      cmdline->step_plane = atoi(optarg);
            }
	  if (i=='f')
            {
	      cmdline->single_frame = atoi(optarg);
	    }
	  if (i=='t')
            {
	      cmdline->sglfr_th0cor_FLAG = 1;
	    }
	  
	  if (i=='r')
	    {
	      cmdline->resol=atoi(optarg);
	      stat_res=1;
	    }
	  if (i=='s')
	    {
	      cmdline->sigma=atof(optarg);
	      cmdline->SIGMA_FLAG=1;
	    }
	  if (i=='I')
	    {
	      cmdline->INTERP_FLAG=1;
	    }
	  if (i=='i')
	    {
	      cmdline->no_iter_rec=atoi(optarg);
	    }
	  
	  
	}
    }
  ct=optind;
  if ((argc-optind < 3) || (argc-optind > 3))
    {
      fprintf(stderr,"\nError: Wrong number of arguments.\n\a\n");
      cmdline_error();
    }
  
  strcpy(cmdline->cfg_file,argv[ct]);
  ct++;

  strcpy(cmdline->init_file,argv[ct]);
  ct++;
  
  strcpy(cmdline->outfile,argv[ct]);
  ct++;

  
  
}
